/***********************************************************************
 *
 * Matt Berntsen, Evan Kaiser, Don Frehulfer
 * CSCI479 at Bucknell University, Fall 2004
 *
 * Course Crawler index-search tool MySQL database config file
 *
 * To load this file, first navigate Terminal to this directory.
 * Log into MySQL
 * Type "\. CourseCrawlerSetup.sql" at the prompt, and hit ENTER
 * 
 * For this to work, you must have MYSQL Connector/J installed.
 * Documentation is available here:
 * http://dev.mysql.com/doc/connector/j/en/index.html
 ***********************************************************************/

CREATE TABLE ccrawler_visited (
	/* Tracks URLs that have been visited. */
	visited_url VARCHAR(200) NOT NULL PRIMARY KEY
) TYPE = InnoDB;

 /* Table used to store sites to be crawled. */
CREATE TABLE ccrawler_sites (
	/* Site Name */
 	site_name VARCHAR (100) NOT NULL,

	/* Site URL */
	site_url VARCHAR(100) NOT NULL,

	/* Allow links to higher levels in URL? */
	site_allow_higher TINYINT DEFAULT 0,

	/* The markup  preceeding a word being defined */
	site_preword TEXT NOT NULL,

	/* The markup following a word being defined, can be NULL */
	site_postword TEXT,

	/* The markup preceeding a definition */
	site_predef TEXT NOT NULL,

	/* The markup following a definition */
	site_postdef TEXT NOT NULL,
	UNIQUE(site_name),
	PRIMARY KEY(site_url, site_name)
) TYPE=InnoDB;

/* Table used to store administrative options, password, etc. */
CREATE TABLE ccrawler_admin (
	admin_password VARCHAR (50) NOT NULL,

	/* Minutes until admin logins expire. Must be 0 - 120 */
	admin_expire INT DEFAULT 5,
	
	/* Log Crawls? */
	admin_log INT DEFAULT 0,

	/* Timestamp of last login. Used in validating admin users. */
	admin_last TIMESTAMP,

	admin_last_crawl DATE,

	admin_last_clean DATE,
	
	/* Flag used for validating admin users. */
	admin_current INT DEFAULT 0,

	admin_cur_ip VARCHAR (16),

	/* Crawl how often? In days. 1 - 365 */
	admin_freq INT DEFAULT 30,

	PRIMARY KEY(admin_password)
) TYPE=InnoDB;

/* Initialize administrative values */
INSERT INTO ccrawler_admin 
	(admin_password, admin_expire, admin_log, admin_current, admin_freq, 
	admin_cur_ip, admin_last_clean)
	VALUES ('admin', 10, 0, 0, 60, '000.000.000.000', CURRENT_DATE());
	
/* Table used in searching */
CREATE TABLE ccrawler_search (
	/* The word being searched for */
	search_word VARCHAR (100),
	
	/* A single site's definition of the word. */
	search_def TEXT NOT NULL,

	/* The name of the site who's definition this is */	
	search_name VARCHAR(100) NOT NULL,
	
	/* The URL that the definition came from */
	search_source TEXT NOT NULL,
	
	/* The definitions rank. */
	search_rank INT UNSIGNED DEFAULT 0,

	/* Date stamp used for data integrity */
	search_date DATE,

	/* Referential Integrity, etc.*/
	INDEX (search_name),		
	PRIMARY KEY (search_word, search_name),
	FOREIGN KEY (search_name) REFERENCES ccrawler_sites(site_name) 
		ON DELETE CASCADE ON UPDATE CASCADE
) TYPE=InnoDB;

/* Table used to determine crawler and parser states */
CREATE TABLE ccrawler_crawl(
	/* 0 if crawler not running, 1 otherwise */
	crawl_crawler int DEFAULT 0,
	/* 0 if parser not running, 1 otherwise */
	crawl_parser int DEFAULT 0,
	PRIMARY KEY (crawl_parser, crawl_crawler)
) TYPE=InnoDB;
	
/* Load in initial values. */
INSERT INTO ccrawler_crawl (crawl_parser, crawl_crawler) VALUES (0, 0);
