
import java.lang.reflect.Array;
import java.net.*;
import java.io.*;
import java.util.*;

import Html_Engine.*;

/*
 * <em>CourseWebServer</em> is a simple web server that can provide limited
 * service. Specifically, it can return one of the three pages:<p>
 * <ol>
 *   <li> the main home page which contains a couple real links,
 *   <li> a <em>sample.html</em> page which contains other forms of html
 *   components such as images and lists, and
 *   <li> a <em>search</em> page that can take input data from the browser
 * </ol>
 *
 * Original Author Xiannong Meng xmeng@bucknell.edu
 * Extended by Team CourseCrawler, Don Frehulfer, Matthew Berntsen, Evan Kaiser 
 *
 * Originally Written Fall 2002
 * Expanded Fall 2004
 *
 */
public class CourseWebServer 
{

    //Pre:  args contains a string which is the port number
    //Post: the server is started on the port specified
    public static void main(String[] args) throws IOException
    {
		if (args.length != 1){
			System.out.println("usage java CourseWebServer port");
			System.exit(-1);
		}
		
		HOME_PAGE = createHomePage();
		ERROR_404 = create404Page();
		ERROR_400 = create400Page();
		objSQL = new cc_db();
		lwLog = new LogWriter();
		pageFunc = new DynamicPages(SERVER_NAME, objSQL);
		
		s = new Semaphore(1);
		list = new PageContentList(s);
		c = new Crawler(list);
		p = new Parser(list);
		ct = new Timer();
		pt = new Timer();
		
		updateCrawler();

		PORT = Integer.parseInt(args[0]);
		/* open a server socket at the given port */
		ServerSocket server = new ServerSocket(PORT);
	
		String inBuffer;
	
		/* the server is running forever (when not crashed :0) */
		for (;;) 
		{
		    try 
		    {
			/* accept an incoming connection request */
			Socket sock = server.accept();
			String ipAddress = sock.getInetAddress().toString();
			ipAddress = ipAddress.substring(1,ipAddress.length());
	
	                /* prepare for input and output stream */
	                BufferedReader sockIn = new BufferedReader
			                          (new InputStreamReader
	                                          (sock.getInputStream()));
			DataOutputStream sockOut = 
	                    new DataOutputStream(sock.getOutputStream());
	
			inBuffer = readAll(sockIn);
	
			http_data = Http.ParseToken(inBuffer);
	
			// echo the command, for debug purposes
			//System.out.print("inBuffer : " + inBuffer);
			//System.out.println();
			//System.out.println("!"+http_data.get("content-length")+"!");
	
			// now process the command, return results when required
			processHTTPCmd(inBuffer, sockIn, sockOut, ipAddress);
	
			/* close the connection */
			sock.close();
		    } catch (Exception ignored) {  }
		}
    }

    //Pre:  none
    //Post: the 400 error page is created;
    private static Page create400Page() {
        Page page = new Page("Error:  Server Couldn't Understand Request");
        page.add("<h1>Error:  Server couldn't understand request!<h1>You will be redirected home automatically.");
	page.setJavaScript("function time() { self.setTimeout(\"window.location = '/'\",3000); }");
	page.setOnBodyLoad("time();");
        return page;
    }

    //Pre:  none
    //Post: the 404 error page is created
    private static Page create404Page() {
        Page page = new Page("Page Not Found");
        page.add("<h1>Error 404:  Page not found</h1><br><br>Please check the URL entered.<br>You will be redirected home automatically.");
	page.setJavaScript("function time() { self.setTimeout(\"window.location = '/'\",3000); }");
	page.setOnBodyLoad("time();");
        return page;
    }

    //Pre:  none
    //Post: the home page is created
    private static Page createHomePage() {
        Page page = new Page("CourseCrawler Main Search");
        Form mainForm = new Form("GET", "/search");
        Input textInput = new Input("text","query","");
        Input submitBtn = new Input("submit","submit","Search");
        Input googleBtn = new Input("submit","submit","Search Google");
	Input extended = new Input("checkbox","related","1");
      
        page.addLink("stylesheet","result.css");
	page.setJavaScript("function sf() { document.f.query.focus(); }");
	googleBtn.setOnClick("window.location = 'http://www.google.com/search?hl=en&q=' + query.value + '&btnG=Search+Google'; return false;");
	mainForm.setCSSClass("input");
        mainForm.add("<center>");
        mainForm.add("Enter search text:");
        mainForm.add(textInput);
        mainForm.add(submitBtn);
	mainForm.add(googleBtn);
	mainForm.setName("f");
	page.setOnBodyLoad("sf();");
	extended.checked(true);
	mainForm.add(extended);
	mainForm.add("<label for=\"related\">Include Similar</label>");
        mainForm.add("</center>");
        
        page.add(mainForm);
                        
        return page;
    }


    /*
     * Reads and returns all the information from a client (browser).
     * This typically is a request (get or post) from a client.<p>
     *
     * @param fileIn a pointer to the buffered reader where the client 
     *               sends in the information.
     * @author Xiannong Meng
     */
    public static String readAll(BufferedReader fileIn)
    {
       String inBuffer = "";
       try
       {

          String line;
	  do 
	  {
              line = fileIn.readLine();
	      inBuffer += line + "\r\n";
	      //	  System.out.print("in readAll -> " + line + "<-");
              if (line.length() == 2 && line.compareTo("\r\n") == 0)
              {
                //System.out.println("we hit the break in readAll");
                break;
              }
	  } while (line.length() > 0);
	}
	catch (Exception ignored) {};
	  //System.out.println("\n end of readAll -> " + inBuffer + "<-");
	return inBuffer;
    }

    /*
     * Process the request (cmd) by a client. Determine what to
     * do based on the parsing result. <p>
     * 
     * The results are directly written to the output stream back
     * to the client.<p>
     *
     * @param cmd the client request
     * @param sockIn socket input stream
     * @param sockOut socket output stream
     *
     * Origninal author Xiannong Meng
     * Extended by Team Course Crawler
     *
     */

    public static void processHTTPCmd(String cmd,
                                      BufferedReader sockIn,
                                      DataOutputStream sockOut,
				      String ipAddress)
    {
	String operation;       // the operation requested
	String path;            // the path to the HTML doc
	String version;         // version of http
	int    returnStatus = 200;   
	                        // status of operation
	long   returnLength = 0;   
                                // length of the doc to return
	boolean isSearch = false; // by default it is not a form input
	boolean adminForm = false; //administration form
	boolean isRank = false; //rank editting
	boolean shouldLog = objSQL.cc_admin_list()[1][1].toString().equals("1");

	/* take the input command and split it into tokens */
	/* here is an example of the command               */
        /* get /path http/1.0 \r\n                         */
	StringTokenizer tokens = new StringTokenizer(cmd, " \r\n");
	operation = (tokens.nextToken()).toUpperCase();
	path      = tokens.nextToken();
	version   = (tokens.nextToken()).toUpperCase();

	/* for debugging purpose */
	//System.out.println("operation: " + operation);
	//System.out.println("path: " + path);
	//System.out.println("version: " + version);

	String path2 = new String(""); //necessary to refer to file without '/'
	for(int i=0; i < path.length(); i++){
		if(path.charAt(i) != '/')
			path2 += path.charAt(i);
	}

	/* check for a request that we dont' understand */
	if ((operation.compareTo("GET") != 0 && 
	    operation.compareTo("POST") != 0) || 
	    ((version.compareTo("HTTP/1.0") != 0) 
	     && (version.compareTo("HTTP/1.1") != 0)))
	    {
	    returnStatus = 400;
	    returnLength = ERROR_400.toHtml().length();
	    }
	else {/* the request is trying to get some HTML page */	

	    if (path.compareTo("/") == 0){
		returnStatus = 200;
		returnLength = HOME_PAGE.toHtml().length();
	    }
	    else if (path.indexOf("/search") >= 0) {
	        isSearch = true;
	    }
	    else if(path.indexOf("/rank") >= 0){
		isRank = true;
	    }
	    else if (path.indexOf("/admin") >= 0 )
	    {
		adminForm = true;
	    }
	    else{
	    	try{
		    fr = new FileReader(path2);
		    File f = new File(path2); //File at "path2" was opened successfully by FileReader
		    returnStatus = 200;
		    returnLength = f.length();
	    	}
	    	catch(FileNotFoundException f){
		    returnStatus = 404;
		    returnLength = ERROR_404.toHtml().length();
		}
	    }
	}
	
	/* now that we know what to do, send the result back */
	if (isSearch)  // this is a form request
        {
	    //processForm(sockIn, sockOut);
	    pageFunc.processSearch(path,sockOut);
	    if(shouldLog)
		lwLog.writeEntry(ipAddress,path);
	}
	else if(isRank){
	    pageFunc.processRankChange(sockOut,path,ipAddress);
	    if(shouldLog)
		lwLog.writeEntry(ipAddress,path);
	}
	else if(adminForm){
	    //processAdmin(sockIn, sockOut, ipAddress, path);
	    processAdminPage(sockIn, sockOut, ipAddress, path);
	    if(shouldLog)
		lwLog.writeEntry(ipAddress,path);
	}
	else
	    try 
	    {
	        if (path.indexOf(".gif") >= 0)
	            sendHeader(sockOut, returnStatus, returnLength, "image/gif");
		else if (path.indexOf(".ico") >= 0)
	            sendHeader(sockOut, returnStatus, returnLength, "image/ico");
	        else if (path.indexOf(".html") >= 0 || path.indexOf(".htm") >= 0)
		    sendHeader(sockOut, returnStatus, returnLength, "text/html");
	        else if (path.indexOf(".jpg") >= 0)
	            sendHeader(sockOut, returnStatus, returnLength, "image/jpg");
	        else if (path.indexOf(".css") >= 0)
	            sendHeader(sockOut, returnStatus, returnLength, "text/css");
	        else if (path.indexOf(".txt") >= 0)
	            sendHeader(sockOut, returnStatus, returnLength, "text/plain");
		else if (!path.equals("/"))
		    returnStatus = 404;
	        //Could add support for any file/mime types I want here
		switch (returnStatus) 
		{
		case 200:      // send the page requested
			if (path.compareTo("/") == 0)
			    sockOut.writeBytes(HOME_PAGE.toHtml());  // short page
			else{ 
			    openAndSendFile(sockOut, path2);
			    if((path.indexOf(".html") >=0 || path.indexOf(".htm") >=0) && shouldLog)
				lwLog.writeEntry(ipAddress,path);
			}
			break;
		case 400:
		    sockOut.writeBytes(ERROR_400.toHtml());
		    break;
		case 404:
		    sockOut.writeBytes(ERROR_404.toHtml());
		    break;
		}
	    } catch (IOException ignored) {}
	
    }

    //Pre:  inVerbose is a non-null string
    //Post: returns a valid browser decoded string, special characters are
    //      converted from their %## equivalent.
    private static String getVerboseStr(String inCoded) {
	String chars = " !\"#$%&'()*+,-./0123456789:;<=>?@";
	String hex = "";
	String verbose = "";
	int pos;

	for(int i = 0; i < inCoded.length(); i++){
	    if(inCoded.charAt(i) == '%'){
		hex = "";
		hex += inCoded.charAt(i+1);
		hex += inCoded.charAt(i+2);
		pos = Integer.parseInt(hex, 16);
		verbose += chars.charAt(pos-0x20);
		i += 2;
	    }
	    else
		verbose += inCoded.charAt(i);
	}
	return verbose;
    }

    //Pre:  inVerbose is a non-null string
    //Post: returns a valid browser encoded string, special characters are
    //      converted to their appropriate %## equivalent.
    private static String getCodedStr(String inVerbose) {
	String chars1 = " !\"#$%&'()**,-./";
	String chars2 = ":;<=>?@";
	String hex = "";
	String coded = "";

	for(int i = 0; i < inVerbose.length(); i++){
	    if(chars1.indexOf(inVerbose.charAt(i)) >= 0){
		hex = Integer.toHexString(chars1.indexOf(inVerbose.charAt(i))+0x20);
		if(hex.length() > 1)
		    coded += "%" + hex;
		else
		    coded += "%0" + hex;
	    }
	    else if(chars2.indexOf(inVerbose.charAt(i)) >= 0){
		hex = Integer.toHexString(chars2.indexOf(inVerbose.charAt(i))+0x39);
		if(hex.length() > 1)
		    coded += "%" + hex;
		else
		    coded += "%0" + hex;
	      	    }
	    else
		coded += inVerbose.charAt(i);
	}
	return coded;
    }

    //Pre: sockIn contains post data, ipAddress is the IP of the client
    //requesting access, sockOut is the output stream and path is the current
    //request path
    //Post: Determines appropriate action to be taken then displays the admin page
    public static void processAdminPage(BufferedReader sockIn,
				  DataOutputStream sockOut, 
				  String ipAddress, 
				  String path){
	try{
	    String pageName = "";
	    String queryString = "";
	    QueryString qsParser = new QueryString(path);
	    boolean isLoggedIn = objSQL.cc_admin_validate_ip(ipAddress);
	    PostData pdData = new PostData(sockIn, http_data);
	    String strOption = qsParser.getValue("opt").toLowerCase();

	    //pull apart the path and query string
	    if (path.indexOf("?") >= 0){
		pageName = path.substring(0,path.indexOf("?"));
		queryString = path.substring(path.indexOf("?") + 1, path.length());
	    }
	    else
		pageName = path;
	    
	    //Process individual pages
	    if(!isLoggedIn){
				    
		if(!pdData.isEmpty()){
		    String pwd = pdData.getValue("password");
		    if(pwd != null && objSQL.cc_admin_validate_pass(pwd)){
			objSQL.cc_admin_start_session(ipAddress);
			pageFunc.sendAdminPage(sockOut);
		    }
		    else{
			pageFunc.sendLoginPage(sockOut);
		    }
		}
		else{
		    pageFunc.sendLoginPage(sockOut);
		}
	    }
	    else if(strOption.equals("")){
		pageFunc.sendAdminPage(sockOut);
	    }
	    else if(strOption.equals("log")){
		if(objSQL.cc_admin_validate_ip(ipAddress)){
		    objSQL.cc_admin_end_session();
		    pageFunc.sendLoginPage(sockOut);
		}
	    }
	    else if(strOption.equals("del") || strOption.equals("add")){
		if(pdData.isEmpty()){
		    pageFunc.sendErrorPage(sockOut,"<li>Invalid Data, please hit back on your browser and try again<br>");
		    return;
		}
		
		String strSiteName = pdData.getValue("sitename").replace('\"','\'');
		
		if(strOption.equals("del")){
		    objSQL.cc_site_delete(strSiteName);
		    pageFunc.sendAdminPage(sockOut);
		}
		else{
		    String strHigher = pdData.getValue("higher");
		    String strPreWord = getVerboseStr(pdData.getValue("preword")).replaceAll("\"","\\\\\"");
		    String strPostWord = getVerboseStr(pdData.getValue("postword")).replaceAll("\"","\\\\\"");
		    String strPreDef = getVerboseStr(pdData.getValue("predef")).replaceAll("\"","\\\\\"");
		    String strPostDef = getVerboseStr(pdData.getValue("postdef")).replaceAll("\"","\\\\\"");
		    String strURL = getVerboseStr(pdData.getValue("url")).replaceAll("\"","\\\\\"");
		    boolean boolHigher;
		    boolean boolExternal;

		    if(strHigher.equals(""))
			boolHigher = false;
		    else
			boolHigher = true;
		    
		    if(strPreWord.equals("") || strPreDef.equals("") || strPostDef.equals("") || 
		       strURL.equals("") || strSiteName.equals("")){
			pageFunc.sendErrorPage(sockOut,"<li>Error:  Missing entry(s)</li>");
			return;
		    }

		    boolean invalidName = objSQL.cc_site_add(strSiteName,strURL,boolHigher,
							   strPreWord,strPostWord,strPreDef,strPostDef);

		    if (!invalidName){
			pageFunc.sendErrorPage(sockOut,"<li>Name already entered.</li>");
			return;
		    }
		    else{
			pageFunc.sendAdminPage(sockOut);
		    }
		    
		}
		
	    }
	    else if(strOption.equals("edit")){
		if (qsParser == null){
		    pageFunc.sendErrorPage(sockOut,"<li>Error:  Invalid Query</li>");
		}
		pageFunc.sendEditPage(sockOut, qsParser);
	    }
	    else if(strOption.equals("submitedit")){
		if (pdData.isEmpty()){
		    pageFunc.sendErrorPage(sockOut,"<li>Error:  Invalid Post Data</li>");
		}
		else{
		    String oldSiteName = getVerboseStr(pdData.getValue("oldsitename")).replaceAll("\"","\\\\\"");
		    String strSiteName = getVerboseStr(pdData.getValue("sitename")).replaceAll("\"","\\\\\"");
		    String strHigher = pdData.getValue("higher");
		    String strPreWord = getVerboseStr(pdData.getValue("preword")).replaceAll("\"","\\\\\"");
		    String strPostWord = getVerboseStr(pdData.getValue("postword")).replaceAll("\"","\\\\\"");
		    String strPreDef = getVerboseStr(pdData.getValue("predef")).replaceAll("\"","\\\\\"");
		    String strPostDef = getVerboseStr(pdData.getValue("postdef")).replaceAll("\"","\\\\\"");
		    String strURL = getVerboseStr(pdData.getValue("url")).replaceAll("\"","\\\\\"");
		    boolean boolHigher;

		    if(strHigher.equals(""))
			boolHigher = false;
		    else
			boolHigher = true;

		    if(strPreWord.equals("") || strPreDef.equals("") || strPostDef.equals("") || 
		       strURL.equals("") || strSiteName.equals("") || oldSiteName.equals(""))
			pageFunc.sendErrorPage(sockOut,"<li>Error:  Missing entry(s)</li>");
		    
		    boolean invalidName = objSQL.cc_site_edit(oldSiteName,strSiteName,strURL,boolHigher,strPreWord,
							      strPostWord,strPreDef,strPostDef);

		    if(!invalidName)
			pageFunc.sendErrorPage(sockOut,"<li>Error:  Invalid Data</li>");
		    else
			pageFunc.sendAdminPage(sockOut);
		}
	    }
	    else if(strOption.equals("passchange")){
		if (pdData.isEmpty()){
		    pageFunc.sendErrorPage(sockOut,"<li>Error:  Invalid Post Data<br>");
		}
		pageFunc.processLoginChange(sockOut,pdData);
	    }
	    else if(strOption.equals("admin")){
		pageFunc.processAdminOpt(sockOut,pdData);
		updateCrawler();
	    }
	    else if(strOption.equals("clean")){
		pageFunc.processClean(sockOut,pdData);
	    }
	    else if(strOption.equals("kill")){
		System.out.println("SERVER TERMINATED BY USER");
		p.cancel();
		c.cancel();		
		System.exit(0);
	    }
	}catch (Exception e) {  e.printStackTrace(); }
    }
    
    //Pre:  none
    //Post: the crawler scheduling is updated to the current frequency
    private static void updateCrawler(){
	Object[][] adminlist = objSQL.cc_admin_list();
	String strFreq = adminlist[1][2].toString();
	int intFreq = Integer.parseInt(strFreq);
	long longFreq;

	if(intFreq > 0)
	    longFreq = intFreq * 86400000;
	else
	    longFreq = 1 * 86400000;

	try{
	    ct.cancel();
	    pt.cancel();
	    c.cancel();
	    p.cancel();
	    s = new Semaphore(1);
	    list = new PageContentList(s);
	    c = new Crawler(list);
	    p = new Parser(list);
	    ct = new Timer();
	    pt = new Timer();
	}catch(Exception e) { System.out.println("Can't Cancel!  Crawler/Parser starting for the first time."); }

	objSQL.cc_crawl_set_crawler(false);
	objSQL.cc_crawl_set_parser(false);

	ct.schedule(c, 5000, longFreq);
	pt.schedule(p, 10000, longFreq);
    }

    /*
     * Send the returning header to the client, based on the return
     * status. The response must follow the HTTP protocol.<p>
     *
     * @param out output stream of the socket
     * @param returnStatus the numerical code for the returning status
     * @param contentLength the length of the content (not the header)
     * @param contentType the type of the return content
     *
     * @author Xiannong Meng
     *
     */
    static void sendHeader(DataOutputStream out, 
			   int returnStatus,
			   long contentLength,
                           String contentType)
    {
	String statusStr;
	String returnContent;

	switch (returnStatus)
	{
	case 200: 
	    statusStr = "OK";
	    break;
	case 400: 
	    statusStr = "Bad Request";
	    break;
	case 404: 
	    statusStr = "Not Found";
	    break;
	default:  
	    statusStr = "Unknown";
	    break;
	}
	
	/*
	 * send an HTTP/1.0 response with Server, Content-Length,
	 * and Content-Type headers.
	 */
        try
        {
	returnContent = "HTTP/1.0 " + returnStatus + " " + statusStr + "\r\n";
	out.writeBytes(returnContent);

	returnContent = "Server: " + SERVER_NAME + "\r\n";
	out.writeBytes(returnContent);

	returnContent = "Content-Length: " + contentLength + "\r\n";
	out.writeBytes(returnContent);

	returnContent = "Content-Type: " + contentType + "\r\n";
	out.writeBytes(returnContent);

	returnContent = "\r\n";
	out.writeBytes(returnContent);
        }
        catch (IOException ignored){}
    }


    /*
     * Send the file requested to the output stream (client).<p>
     * 
     * @param out output stream to the client.
     * @param fileToSend string name of the file to be sent.
     *
     * @author Xiannong Meng
     *
     */
    static void openAndSendFile(DataOutputStream out, String fileToSend)
    {
	try
	{
	    DataInputStream fileIn = new DataInputStream
					   (new FileInputStream
					    (new File(fileToSend)));
	    
            File f = new File(fileToSend);
            long length = f.length();
	    byte[] content = new byte[(int)length];

	    fileIn.readFully(content);
            out.write(content);
	} catch (IOException ignored) {};
    }

    /* this is the class data segments */
    
    static FileReader fr; // For reading a file

    static int PORT;  // the port used for communication

    static Hashtable http_data;  // HTTP header infromation from client
    
    static ArrayList users;

    // name of the server
    static String SERVER_NAME	= "CourseCrawler Web Server";

    // error message 400
    private static Page ERROR_400;

    // error message 404
    private static Page ERROR_404;

    // a simple, short home page
    private static Page HOME_PAGE;

    //instance of the cc_db class, used to access the MySQL server
    private static cc_db objSQL;

    //instance of the LogWriter class, used to write web site usage logs
    private static LogWriter lwLog;

    private static DynamicPages pageFunc;

    //Stuff used by the parser/crawler
    private static Semaphore s;
    private static PageContentList list;
    private static Crawler c;
    private static Parser p;
    private static Timer ct;
    private static Timer pt;
}
