import java.io.*;
import java.util.*;
import java.sql.*;
import java.net.*;
import java.lang.reflect.Array;

import javax.swing.JOptionPane; //for debug

/*
 * Created on Oct 12, 2004
 * By Evan Kaiser, Don Frehulfer, and Matt Berntsen
 * CS 479 - Capstone Design
 */

//public class Crawler extends Thread{
public class Crawler extends TimerTask{
    public final static int PORT = 80; //default web port
    public final static String protocol = "http://";
    public final static String protocolS = "https://";
    public final static String protocolMail = "mailto:";
    public final static String protocolFTP = "ftp://";
    public final static String protocolAIM = "aim:";
    
    private cc_db db;              //CourseCrawler database
    private PageContentList pList; //Parser to find terms and definitions
    private Object[][] siteList;   //List of sites obtained from database
    private String[] childPages;   //Links to child pages from current page
    private InputStream input;     //Stream from online page
    private DataInputStream inPage;//Current page looked at
    private BufferedWriter outPage;//Temporary page to write to
    private FileReader page;       //Page on disk to parse
    private HttpURLConnection uC;  //Connection to current url
    private URL u; //URL is in the form "http://www.yahoo.com/"

    //Values obtained for definition parsing
    private String preterm, postterm, predef, postdef;
    private int allowHigher;

    //List of links stored for Crawler to parser through
    private ArrayList linkNameList, linkURLList, disallowedURLList;


    //Pre:  inList is a reference to a PageContentList created in RunCrawler.
    //Post: db gets instantiated. pList gets set to inList.
    //      siteList gets set to the return value of cc_db::cc_site_list.
    //      linkNameList, linkURLList, and disallowedURLList are instantiated.
    public Crawler(PageContentList inList){
	db = new cc_db();
	pList = inList;
	siteList = db.cc_site_list();
	linkNameList = new ArrayList(0);
	linkURLList = new ArrayList(0);
	disallowedURLList = new ArrayList(0);
	allowHigher = 0;
    }

    //Pre: linkURL has a value that includes a domain with a possible path
    //Post: return the domain only without a '/' at the end
    private String getDomain(String linkURL){
	String domain = "";
	int start = 0;
	if(linkURL.indexOf("://") >= 0)
	    start = linkURL.indexOf("://") + 3;
	for(int i = start; i < linkURL.length(); i++){
	    if(linkURL.charAt(i) != '/')
		domain += linkURL.charAt(i);
	    else
		break;
	}
	return domain;
    }

    //Pre: inURL has a value with no protocol
    //Post: if page is disallowed, add to visited list in database
    private boolean checkRobotsTXT(String inURL){
	String robotsURL = "http://" + getDomain(inURL) + "/robots.txt";
	//System.err.println("Robots.txt: "+robotsURL);
	boolean more = true;
	boolean foundUserAgent = false;
	String str, page;
	BufferedReader robotsPage;
	InputStream in;
	Integer responseCode = new Integer(0);
	connectToSite(robotsURL, responseCode);
	if(responseCode.intValue() != 404){
	    try{
		in = uC.getInputStream();
		robotsPage = new BufferedReader(new InputStreamReader(in));
		while(more){ // read until no more
		    str = robotsPage.readLine();
		    //System.err.println(str);
		    if (str == null)
			more = false;
		    else{
			if(str.indexOf("User-agent:") >= 0 && str.indexOf("*") >= 0){
			    foundUserAgent = true;
			    //System.err.println("Found user-agent.");
			}
			else if(foundUserAgent && (str.indexOf("Disallow: ") == 0)){
			    page = str.substring(10, str.length()-1);
			    disallowedURLList.add(getDomain(inURL) + page);
			    //System.err.println("Adding page " + getDomain(inURL) + page + " to disallow list.");
			}
		    }
		}
	    }
	    catch (IOException ex){
		return true; //Couldn't read from robots.txt for some reason
	    }
	    return true;
	}
	else{
	    //System.err.println("Site doesn't contain a robots.txt");
	    return false; //Site doesn't have a robots.txt page
	}
    }

    //Pre:  inURL has a value of form http://domain.ext/optional_path.
    //Post: Return true if connection was made.
    private boolean connectToSite(String inURL, Integer rCode){
	int responseCode = 0;
	
	try{
	    u = new URL(inURL);
	    //	    uC.setChunkedStreamingMode(32);
	    // connet to the site
	    uC = (HttpURLConnection)u.openConnection();
	    uC.setInstanceFollowRedirects(false);
	    responseCode = uC.getResponseCode();
	    rCode = new Integer(responseCode);
	}
	catch (Exception ex){
	    return false;
	}
	
	//System.err.println("HTTP Status-Code = " + responseCode);

	if (responseCode != HttpURLConnection.HTTP_OK){
	    return false;
	}
	return true;
    }

    //Pre:  inURL has a value with a protocol (http://)
    //Post: Return main domain (i.e. domain.ext) without subdomains such as "www"
    private String getMainDomain(String inURL){
	String urlDomain = ""; //domain (without subdomains) of inURL
	int i = 0;
	int slashPos = 0; //position of first slash after 'http://'
	int dotPos = 0; //position of dot before real domain starts
	boolean dotFound = false;

	i = inURL.indexOf("://") + 3;
	while(i < inURL.length()){
	    if(inURL.charAt(i) == '/' && slashPos == 0)
		slashPos = i;
	    i++;
	}
	
	if(slashPos == 0){
	    slashPos = i-1;
	}
	
	i = slashPos;
	
	while(i >= inURL.indexOf("://")){
	    if(!dotFound && inURL.charAt(i) == '.') //found dot before extension
		dotFound = true;
	    else if(dotFound && inURL.charAt(i) == '.'){ //found dot before real domain
		dotPos = i;
		dotFound = false;
		break;
	    }
	    else if(dotFound && i == inURL.indexOf("://") + 2){
		dotPos = i;
		dotFound = false;
		break;
	    }
	    i--;
	}
	
	urlDomain = inURL.substring(dotPos+1, slashPos);

	return urlDomain;
    }

    //Pre:  inLink has a value.
    //Post: Return true if link is in outside domain and
    //      return false if link is in same domain.
    private boolean checkLinkExternal(String inLink, String inCurURL){
	String linkDomain = ""; //domain (without subdomains) of inLink
	String curURLDomain = ""; //domain (without subdomains) of inCurURL
	int i = 0; //counter for inLink
	int j = 0; //counter for inCurURL
	int slashPos1 = 0, slashPos2 = 0; //position of first slash after 'http://'
	int dotPos1 = 0, dotPos2 = 0; //position of dot before real domain starts
	boolean dotFound = false;
	
	if(inLink.indexOf("://") >= 0){
	    linkDomain = getMainDomain(inLink);
	    curURLDomain = getMainDomain(inCurURL);
	    
	    if(linkDomain.equals(curURLDomain))
		return false;
	    else
		return true;
	}
	else
	    return false;
    }

    //Pre:  inLink has a value.
    //Post: Return true if link has a valid protocol.
    private boolean checkValidProtocol(String inLink){
	if(inLink.indexOf(protocol) >= 0)
	    return true;
	else
	    return false;
    }

    //Pre:  inURL has a value that contains the full path and page.
    //Post: Return true if it contains a valid page extension
    private boolean checkValidPage(String inURL){
	String temp = inURL;
	int i = 0;

	if(temp.indexOf(protocol) == 0)
	    temp = temp.substring(protocol.length(), inURL.length());

	while(i < temp.length() && temp.charAt(i) != '/')
	    i++;
	
	temp = temp.substring(i, temp.length());

	//System.err.println("Checking " + temp + " for valid page.");
	if(temp.indexOf(":") > 0){//nested protocol - return false
	    return false;
	}
	else if(temp.indexOf(".") >= 0){ //occurence of '.' after first '/' so link has
	                                //to have a page with an extension
	    
	    if(temp.indexOf(".htm") > 0 || temp.indexOf(".asp") > 0 ||
	       temp.indexOf(".cgi") > 0 || temp.indexOf(".php") > 0 ||
	       temp.indexOf(".xml") > 0 || temp.indexOf(".cfm") > 0)
		return true;
	    else
		return false;
	}
	else
	    return true;
    }

    //Pre:  inLinkName and inLinkURL have values.
    //Post: Add items to corresponding lists.
    private void addLink(String inLinkName, String inLinkURL){
	linkURLList.add(inLinkURL);
	linkNameList.add(inLinkName);
    }

    //Pre:  inURL has a value without domain. curURL has a value of the current
    //      page being parsed.
    //Post: Returns true if link moves to higher directory or if it is an
    //      absolute link that's not the same directory as curURL.
    private boolean checkLinkHigher(String inURL, String curURL){
	int i;
	if(inURL.charAt(0) == '/'){ //absolute link
	    int k;
	    for(k = curURL.length()-1; k > 0; k--){
		if(curURL.charAt(k) == '/')
		    break;
	    }
	    String path = curURL.substring(0, k); 
	    if(inURL.indexOf(path) >= 0)
		return false;
	    else
		return true;
	}
	else if(inURL.indexOf("../") >= 0) //moves up a directory
	    return true;
	else
	    return false;
    }

    //Pre:  inURL has a value that possibly contains "./" or "../".
    //Post: Return a URL with occurences of "./" changed to just "/"
    //      and occurences of "../" processed into moving up one directory.
    private String translateURL(String inURL){
	//System.err.println(inURL);
	String str = inURL.replaceAll("/\\p{Punct}/", "/");
	String url = "";
	boolean noPage = true; //no page is included in url
	ArrayList dirStack = new ArrayList(0); //stack to process directories and "../"
	String curDir;
	String page = "";

	str = str.replaceAll(" ", "");
	String domain = getDomain(inURL);
	dirStack.add(domain);
	str = str.replaceAll("//", "/");
	for(int j = str.indexOf(domain) + domain.length(); j < str.length(); j++){
	    curDir = "";
	    //System.err.println(j + ":: " + str.charAt(j));
	    if(str.charAt(j) == '/'){
		j++;
		while(j < str.length() && str.charAt(j) != '/'){
		    curDir += str.charAt(j);
		    //System.err.println(curDir);
		    //System.err.println(j + ": " + str.charAt(j));
		    j++;
		}
		//System.err.println(str.charAt(j));
		if(j < str.length() && str.charAt(j) == '/'){//found a directory
		    //System.err.println(curDir);
		    if(curDir.equals("..")){
			if(dirStack.size() > 1){//if only domain, ignore "../"
			    dirStack.remove(dirStack.size()-1); //pop previous directory
			}
		    }
		    else{
			//System.err.println(curDir);
			dirStack.add(curDir); //push current directory
		    }
		}
		else //found a page
		    page = curDir;
		j--;
	    }
	}
	//turn dirStack into a FIFO queue and concatenate entries onto new URL, 'url'
	for(int k = 0; k < dirStack.size(); k++){
	    //System.err.println(dirStack.get(k));
	    url += (String)dirStack.get(k) + "/";
	}
	if(page.length() > 0)
	    url += page;
	return url;
    }

    //Pre:  inName and inURL have values.
    //Post: Parse for links and add them to linkNameList and linkURLList
    //      and also add a PageContent to the PageContentList.
    private void parseLinks(String inName, String inURL){
	int count = 0;
	int length = 0;
	int c;
	boolean more = true;
	boolean gotToLinkName = false;
	boolean nameDone = false;
	String lengthStr;
	String str = "";
	String startA = "<a ";
	String startHref = "href=";
	String endA = "</a>";
	String tag = "";
	String linkURL = "";

	try{
	    input = uC.getInputStream();
	    lengthStr = uC.getHeaderField("content-length");
	    inPage = new DataInputStream(input);
	    outPage = new BufferedWriter(new FileWriter(".content_tmp"));


	    /***********************************
	     * Copy from socket stream to file *
	     ***********************************/
	    boolean readByLine = false;
	    String temp = "";

	    if(lengthStr != null){
		byte[] data = new byte[length];
		length = Integer.parseInt(lengthStr.trim());
		
		inPage.readFully(data);
		temp = new String(data);
		if(temp.length() == length){
		    temp = temp.replaceAll("\n", " ");
		    temp = temp.replaceAll("\t", " ");
		    temp = temp.replace('\\',' ');
		}
		else
		    readByLine = true;
	    }

	    if(lengthStr == null || readByLine){
		BufferedReader bufPage = new BufferedReader(new InputStreamReader(input));
		temp = "";
		while(more) { // read until no more
		    str = bufPage.readLine();
		    //System.err.println(str);
		    if(str == null)
			more = false;
		    else{
			temp = temp.replaceAll("\t", " ");
			temp = temp + str + " ";
			temp = temp.replace('\\',' ');
		    }
		}
	    }
	    int tempLen = temp.length();
	    while((temp = temp.replaceAll("  ", " ")).length() != tempLen){
		//System.err.println(tempLen);
		//System.err.println(temp.length());
		tempLen = temp.length();
	    }
	    outPage.write(temp);
	    outPage.write('>'); //to avoid infinite loops in case of bad html
	    outPage.flush();

	    /********************************
	     * Look for links               *
	     ********************************/
	    
	    page = new FileReader(".content_tmp");
	    

	    //check for a links
	    while(page.ready()){
		linkURL = "";
		
		/********************************************
		 * Look for linkTag  and determine link URL *
		 ********************************************/
		
		c = page.read();
		if(c == startA.charAt(0)){
		    //System.err.println("startA 0: "+(char)c);
		    int j;
		    for(j = 1; j < startA.length(); j++){
			c = page.read();
			//if(inURL.equals("www.cabletron.com/support/Tools2/sfhelp/html/doc/html/mibPoller.html"))
			//System.err.println("startA "+j+": "+(char)c);
			if(Character.toLowerCase((char)c) != startA.charAt(j))
			    break;
		    }
		    if(j == startA.length() && c == startA.charAt(startA.length()-1)){
			while(((c = page.read()) != startHref.charAt(0)) && (c != '>'));
			int k;
			if(c == startHref.charAt(0)){
			    //if(inURL.equals("www.cabletron.com/support/Tools2/sfhelp/html/doc/html/mibPoller.html"))
			    //System.err.println("startHref 0: "+(char)c);
			    for(k = 1; k < startHref.length(); k++){
				c = page.read();
				//if(inURL.equals("www.cabletron.com/support/Tools2/sfhelp/html/doc/html/mibPoller.html"))
				//System.err.println("startHref "+k+": "+(char)c);
				if(Character.toLowerCase((char)c) != startHref.charAt(k))
				    break;
			    }
			    if(k == startHref.length() && c == startHref.charAt(startHref.length()-1)){
				c = page.read();
				if(c == '\"'){
				    while(page.ready() && ((c = page.read()) != '\"') && 
					  (c != '>') && (c != '#')){
					linkURL += (char)c;
				    }
				}
				else{
				    linkURL += (char)c;
				    while(page.ready() && ((c = page.read()) != ' ') && 
					  (c != '>') && (c != '#') && (c != '\"')){
					linkURL += (char)c;
				    }
				}
				//if(inURL.equals("www.cabletron.com/support/Tools2/sfhelp/html/doc/html/mibPoller.html"))
				// System.err.println("URL: " + linkURL);
			    }
			}
		    }
		    else
			continue;


		    /********************************************
		     * Determine accurate full URL and new name *
		     ********************************************/
		    
		    if(linkURL.length() > 0){
			//if(inURL.equals("www.cabletron.com/support/Tools2/sfhelp/html/doc/html/mibPoller.html"))
			//System.err.println("Got here!");
			if((allowHigher==0 && !checkLinkHigher(linkURL, inURL)) || (allowHigher==1)){
			    if(checkValidProtocol(linkURL) && //full link with protocol
			       !checkLinkExternal(linkURL, "http://"+inURL)){ //and make sure it's not external

				linkURL = translateURL(linkURL);
				
				if(!db.cc_visited_visited(linkURL) && checkValidPage(linkURL)){ 
				    //check if link hasn't already been found
				    //on another page
				    addLink(inName, linkURL);
				    pList.addSite(new PageContent(inName, linkURL, preterm,
								  postterm, predef, postdef));
				}
				db.cc_visited_add(linkURL);
			    }
			    else if(!checkValidProtocol(linkURL) && //not a full link
				    !checkLinkExternal(linkURL, "http://"+inURL)){ //or external link
				int k;
				for(k = inURL.length()-1; k > 0; k--){
				    if(inURL.charAt(k) == '/')
					break;
				}
				String path = inURL.substring(0, k);
				
				if(linkURL.charAt(0) != '/')
				    linkURL = path + '/' + linkURL;
				else
				    linkURL = getDomain(path) + linkURL;
				
				linkURL = translateURL(linkURL);
				
				//System.err.println(inURL + ": " + linkURL);
				
				if(!db.cc_visited_visited(linkURL) && checkValidPage(linkURL)){ 
				    //check if link hasn't already been
				    //found on another page
				    //if(inURL.equals("www.cabletron.com/support/Tools2/sfhelp/html/doc/html/mibPoller.html"))
				    //System.err.println("Full URL: "+linkURL);
				    addLink(inName, linkURL);
				    pList.addSite(new PageContent(inName, linkURL, preterm,
								  postterm, predef, postdef));
				}
				db.cc_visited_add(linkURL);
			    }
			}
		    }
		}
	    }
	    page.close();
	}
	catch (IOException ex){
	    return;
	}
    }
    
    //Pre: 'this' has a value
    //Post: run Thread 'this'
    public void run(){
	String name;
	String url;
	int pageCount = 0;
	boolean hasRobotsTXT = false;

	//	while(true){
	if(db.cc_admin_crawl() && !db.cc_crawl_crawler_running() && siteList != null){
	    db.cc_visited_clear();
	    db.cc_crawl_set_crawler(true);
	    
	    for(int i = 1; i < Array.getLength(siteList); i++){
		name = (String)siteList[i][0];
		url = ((String)siteList[i][1]);
		allowHigher = ((Integer)(db.cc_site_find(name)[1][2])).intValue();
		preterm = ((String)(db.cc_site_find(name)[1][3])).toLowerCase();
		postterm = ((String)(db.cc_site_find(name)[1][4])).toLowerCase();
		predef = ((String)(db.cc_site_find(name)[1][5])).toLowerCase();
		postdef = ((String)(db.cc_site_find(name)[1][6])).toLowerCase();

		if(url.indexOf(protocol) >= 0){
		    url = url.substring(protocol.length(),url.length());
		}

		//System.err.println(url);

		//if(!url.equals("www.pcwebopedia.com/Top_Category.asp"))
		//    continue;

		hasRobotsTXT = checkRobotsTXT(url);

		if((hasRobotsTXT && !disallowedURLList.contains(url)) || !hasRobotsTXT){
		    if(connectToSite("http://" + url, new Integer(0))){
			//System.err.println("\nFinding links in "+name+" url="+url+"\nWith preterm="+
			//preterm+", postterm="+postterm+", predef="+predef+
			//", and postdef="+postdef);
			pList.addSite(new PageContent(name, url, preterm, postterm, predef, postdef));
			db.cc_visited_add(url);
			parseLinks(name, url);
			pageCount++;
		    }
		    //else
			//System.err.println("Failed to connect to http://" + url);
		}
		//else;
		    //System.err.println("Robots.txt disallowed URL: " + url);

		//System.err.println("Finished finding links in original page.");

		String newLinkURL;
		String newLinkName;
		int maxListSize = 0;
		while((linkNameList.size() > 0) && (linkNameList.size() == linkURLList.size())){
		    newLinkName = (String)linkNameList.remove(linkNameList.size()-1);
		    newLinkURL = (String)linkURLList.remove(linkURLList.size()-1);

		    if((hasRobotsTXT && !disallowedURLList.contains(newLinkURL)) || !hasRobotsTXT){
			if(connectToSite("http://" + newLinkURL, new Integer(0))){
			    //System.err.println("\nFinding links in "+newLinkName+" url="+newLinkURL+
			    //"\nWith preterm="+preterm+", postterm="+postterm+
			    //", predef="+predef+", and postdef="+postdef);
			    parseLinks(newLinkName, newLinkURL);
			    pageCount++;
			}
			//else
			    //System.err.println("Failed to connect to http://" + newLinkURL);
		    }
		    //else
			//System.err.println("Robots.txt disallowed url: " + newLinkURL);

		    //System.err.println("Current page list size (Crawler): "+pList.getSize());
		    //System.err.println("Current link list size: "+linkNameList.size());
		    //System.err.println("Finished finding links in subsequent page.");
		}
	    }
	    System.out.println("Found links in " + pageCount + " pages.");
	    db.cc_crawl_set_crawler(false);
	    db.cc_admin_setCrawl();
	}
	pList.addSite(new PageContent("" + (char)4, "", "", "", "", ""));
	//send EOT character (ASCII value 4) as NAME

	if(db.cc_crawl_crawler_running())
	    System.out.println("Crawler is already running.");
    }
}
