
import java.lang.reflect.Array;
import java.net.*;
import java.io.*;
import java.util.*;

import Html_Engine.*;

//Class that generates all dynamic pages
public class DynamicPages {

    private cc_db objSQL;
    private String SERVER_NAME;
    private ArrayList arrVoters;
    private ArrayList arrTimes;
    private int intBegin;
    private int intEnd;
    private int intPageNumber;
    
    //Pre:  inServerName is a string that represents the server name, inSQL is
    //      an active SQL Server connection (thru cc_db class)
    //Post: DynamicPages is initialized
    public DynamicPages(String inServerName, cc_db inSQL){
	objSQL = inSQL;
	SERVER_NAME = inServerName;
	arrVoters = new ArrayList();
	arrTimes = new ArrayList();
    }

    //Pre:  inCoded is a non-null string
    //Post: returns a valid browser decoded string, special characters are
    //      converted from their %## equivalent.
    private  String getVerboseStr(String inCoded) {
	String chars = " !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ" + 
		       "[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~";
	String hex = "";
	String verbose = "";
	int pos;

	for(int i = 0; i < inCoded.length(); i++){
	    if(inCoded.charAt(i) == '%'){
		hex = "";
		hex += inCoded.charAt(i+1);
		hex += inCoded.charAt(i+2);
		pos = Integer.parseInt(hex, 16);
		verbose += chars.charAt(pos-0x20);
		i += 2;
	    }
	    else
		verbose += inCoded.charAt(i);
	}
	return verbose;
    }

    //Pre:  inVerbose is a non-null string
    //Post: returns a valid browser encoded string, special characters are
    //      converted to their appropriate %## equivalent.
    private  String getCodedStr(String inVerbose) {
	String chars1 = " !\"#$%&'()**,-./";
	String chars2 = ":;<=>?@";
	String hex = "";
	String coded = "";

	for(int i = 0; i < inVerbose.length(); i++){
	    if(chars1.indexOf(inVerbose.charAt(i)) >= 0){
		hex = Integer.toHexString(chars1.indexOf(inVerbose.charAt(i))+0x20);
		if(hex.length() > 1)
		    coded += "%" + hex;
		else
		    coded += "%0" + hex;
	    }
	    else if(chars2.indexOf(inVerbose.charAt(i)) >= 0){
		hex = Integer.toHexString(chars2.indexOf(inVerbose.charAt(i))+0x39);
		if(hex.length() > 1)
		    coded += "%" + hex;
		else
		    coded += "%0" + hex;
	      	    }
	    else
		coded += inVerbose.charAt(i);
	}
	return coded;
    }

    //Pre:  out is an active DataOutputStream, returnstatus is a valid html
    //      return code, contentlength is the length of the page to send, and content
    //      type is a valid mime type
    //Post: sends an appropriate html header
    private void sendHeader(DataOutputStream out, 
			   int returnStatus,
			   long contentLength,
                           String contentType)
    {
	String statusStr;
	String returnContent;

	switch (returnStatus)
	{
	case 200: 
	    statusStr = "OK";
	    break;
	case 400: 
	    statusStr = "Bad Request";
	    break;
	case 404: 
	    statusStr = "Not Found";
	    break;
	default:  
	    statusStr = "Unknown";
	    break;
	}
	
	/*
	 * send an HTTP/1.0 response with Server, Content-Length,
	 * and Content-Type headers.
	 */
        try
        {
	returnContent = "HTTP/1.0 " + returnStatus + " " + statusStr + "\r\n";
	out.writeBytes(returnContent);

	returnContent = "Server: " + SERVER_NAME + "\r\n";
	out.writeBytes(returnContent);

	returnContent = "Content-Length: " + contentLength + "\r\n";
	out.writeBytes(returnContent);

	returnContent = "Content-Type: " + contentType + "\r\n";
	out.writeBytes(returnContent);

	returnContent = "\r\n";
	out.writeBytes(returnContent);
        }
        catch (IOException ignored){}
    }

    //Pre:  sockOut is a socket connection to the browser, path is a valid web
    //      path, and ipAddress is the IP of the voter
    //Post: if the voter is allowed, casts a vote for the specified term
    public void processRankChange(DataOutputStream sockOut, String path, 
				  String ipAddress){
	try{
	    QueryString qs = new QueryString(path);
	    String option = qs.getValue("opt");
	    String sitename = getVerboseStr(qs.getValue("sitename"));
	    String term = getVerboseStr(qs.getValue("term"));
	    String query = qs.getValue("query");
	    String related = qs.getValue("related");
		String pageNumber = qs.getValue("page");
	    boolean success = false;
	    Page page = new Page("Page Rank");
	    GregorianCalendar current = new GregorianCalendar();

	    
	    Link lnkReturn = new Link();
	    lnkReturn.setHref("/search?query=" + query + "&related=" + related + "&page=" + pageNumber);
	    lnkReturn.add("Return to results");

	    cleanDates();

	    if(arrVoters.indexOf(ipAddress) >= 0){
	
		    page.add("<h2>You have voted too recently, users may only vote once every 8 hours.  <br>Try again later!</h2>");
	    }
	    else{

		current.add(Calendar.HOUR,8);
		arrTimes.add(current);
		arrVoters.add(ipAddress);
       
		if(option.equals("up"))
		    success = objSQL.cc_search_edit_weight(term,sitename,1);
		else
		    success = objSQL.cc_search_edit_weight(term,sitename,-1);
		
		if(success){
		    page.add("<h2>Rank modified successfully</h2>");
		}
		else
		    page.add("<h2>Rank modification unsuccessful.  Bad Data.</h2>");
	    }
	    page.add(lnkReturn);
	    page.setJavaScript("function time() { self.setTimeout(\"window.location = '/search?query=" + query
			       + "&related=" + related + "&page=" + pageNumber + "'\",3000); }");
	    page.setOnBodyLoad("time();");
	    sendHeader(sockOut, 200, page.toHtml().length(), "text/html");
	    sockOut.writeBytes(page.toHtml());
	    
	}catch (IOException e){ e.printStackTrace(); }
    }

    //Pre:   none
    //Post:  Any voter records older than the current time are removed from the
    //       list
    private void cleanDates(){
	GregorianCalendar current = new GregorianCalendar();
	arrTimes.trimToSize();
	arrVoters.trimToSize();
	int size = arrTimes.size();

	for(int i = 0; i < arrTimes.size(); i++){
	    if(current.after(arrTimes.get(i))){
		arrTimes.remove(i);
		arrVoters.remove(i);
		i--;
	    }
	}
    }

    //Pre:  arrResults is a non-null array of results, pageNumber is a non-null
    //      string
    //Post: depending on the pageNumber the results list is truncated to a
    //      specified page (e.g. page 1 -> results 1-10).  Also, page number
    //      begin and end index are recorded in global variables
    private Object[][] pageResults(Object[][] arrResults, String pageNumber){

	int pageNum;
	Object[][] rtnArray;
	int arrLength = Array.getLength(arrResults);
	int rowLength = Array.getLength(arrResults[0]);
	int counter = 0;

	//strips the title row
	try{
	    pageNum = Integer.parseInt(pageNumber);
	}catch (Exception e) { pageNum = 1; }
	
	
	if(pageNum < 1 || arrLength - (10*(pageNum-1)) <= 0){
	    pageNum = 1;
	}
	
	intPageNumber = pageNum;

	if(arrLength >= (10*pageNum)){
	    rtnArray = new Object[10][rowLength];
	    counter = 0;
	    intBegin = (10*pageNum) - 9;
	    intEnd = (10*pageNum);
	    for(int i = (10*pageNum) - 10; i < 10*pageNum; i++){
		rtnArray[counter] = arrResults[i];
		counter++;
	    }
	}
	else {
	    rtnArray = new Object[arrLength - (10*(pageNum-1))][rowLength];
	    counter = 0;
	    intBegin = (10*(pageNum-1)) + 1;
	    intEnd = arrLength;
	    for(int i = (10*(pageNum-1)); i < arrLength; i++){
		rtnArray[counter] = arrResults[i];
		counter++;
	    }
	}
	return rtnArray;
	    
    }

    //Pre:  path is a web address containing a search query, sockOut is a
    //      DataOutputStream for the web connection.
    //Post: the search is run, and a results page is sent to the browser
    public void processSearch(String path, DataOutputStream sockOut) {
        try{
	    QueryString qs = new QueryString(path);
	    String query = getVerboseStr(qs.getValue("query"));
	    String pageNum = getVerboseStr(qs.getValue("page"));
	    int resultLimit;
	    int resultStart;
	    //query = makeAlphaNumeric(query);
	    query = query.replaceAll("%","");
	    query = query.replaceAll("\"","");
	    String related = qs.getValue("related");
            Page page = new Page("Search Results for " + query);
            Form form1 = new Form("GET", "/search");
            Input textInput = new Input("text", "query",query);
            Input submit = new Input("submit","submit","Search");
	    Input google = new Input("submit","submit", "Search Google");
	    Input extended = new Input("checkbox", "related", "1");
	    Label resultCount = new Label();
            DataGrid resultGrid = new DataGrid();
            Link defLink = new Link();
	    String htmlPage;
	    Link lnkCredits = new Link();

	    lnkCredits.setHref("credits.html");
	    lnkCredits.setCSSClass("creditLink");
	    lnkCredits.add("Site Credits");

	  
	    extended.checked(related.equals("1"));
            
	    google.setOnClick("window.location = 'http://www.google.com/search?hl=en&q=' + query.value + '&btnG=Search+Google'; return false;");

	    page.setJavaScript("function sf() { document.f.query.focus(); } ");
	    page.setOnBodyLoad("sf();");
	    form1.add("Enter search text:");
            form1.add(textInput);
            form1.add(submit);
	    form1.add(google);
	    form1.add(extended);
	    form1.setName("f");
	    form1.add("<label for=\"related\">Include Similar</label>");
	    resultCount.setCSSClass("resultCount");
	    form1.add(resultCount);
            form1.setCSSClass("input");
            
            page.addLink("stylesheet","result.css");
            
            page.add(form1);
            page.add("<h2>Search results for: " + query + "</h2>");
	    
	    if(!query.equals("") && query.length() >= 2){
		Object[][] results;
		//System.out.println("Query: " + query)

		if (related.equals("1"))
		    results = objSQL.cc_search_find(query,true);
		else
		    results = objSQL.cc_search_find(query,false);

		if (results != null){

		    Object[][] newResults = pageResults(results,pageNum);

		    resultCount.setText(intBegin + " - " + intEnd + " of " + Array.getLength(results) + " results");
		    Object[][] display = new Object[Array.getLength(newResults) + 1][4];
		    display[0][0] = "Link";
		    display[0][1] = "Term";
		    display[0][2] = "Definition";
		    display[0][3] = "Was this useful?";
 
		    for(int i = 1; i < Array.getLength(newResults) + 1; i++){
			Link lnkTemp = new Link();
			Input btnRankUp = new Input("submit","","Yes");
			Input btnRankDown = new Input("submit","","No");
			lnkTemp.setHref("http://" + (String)newResults[i-1][3]);
			lnkTemp.setTarget("_blank");
			lnkTemp.add(newResults[i-1][0]);
			btnRankUp.setOnClick("window.location = '/rank?opt=up&sitename=" + newResults[i-1][0] + 
					     "&term=" + newResults[i-1][1] + "&query=" + query + "&related=" + 
					     related + "&page=" + pageNum + "'; return false;");
			btnRankDown.setOnClick("window.location = '/rank?opt=down&sitename=" + newResults[i-1][0] + 
					       "&term=" + newResults[i-1][1] + "&query=" + query + "&related=" + 
					       related + "&page=" + pageNum + "'; return false;");
			display[i][0] = lnkTemp;
			display[i][1] = newResults[i-1][1];
			display[i][2] = newResults[i-1][2];
			display[i][3] = btnRankUp.toString() + "&nbsp;" + btnRankDown.toString();
		    }
		    resultGrid.setFirstRowTitle(true);
		    resultGrid.setDataSource(display);
		    resultGrid.setCSSClass("result");  

		    int numPages = Array.getLength(results);

		    if(numPages % 10 > 0)
			numPages = numPages / 10 + 1;
		    else
			numPages = numPages / 10;
		    
		    page.add("| ");
		    for(int i = 1; i <= numPages; i++){
			if(intPageNumber != i){
			    Link lnkPage = new Link();
			    lnkPage.setHref("/search?query=" + query + "&related=" + related + "&page=" + i);
			    lnkPage.add(i + "");
			    page.add(lnkPage);
			}
			else 
			    page.add("<b>" + i + "</b>");

			    page.add(" | ");
		    }

		    page.add(resultGrid);

		    page.add("| ");
		    for(int i = 1; i <= numPages; i++){
			if(intPageNumber != i){
			    Link lnkPage = new Link();
			    lnkPage.setHref("/search?query=" + query + "&related=" + related + "&page=" + i);
			    lnkPage.add(i + "");
			    page.add(lnkPage);
			}
			else 
			    page.add("<b>" + i + "</b>");

			page.add(" | ");
		    }
		}
		else{
		    page.add("<h2>No Matches Found</h2>");
		}
	    }
	    else
		page.add("<h2>Invalid Entry!</h2><li>Query must be at least 2 characters!</li>");
	    
	    page.add("<hr>");
	    page.add("<center>" + lnkCredits + "</center>");
	    htmlPage = page.toHtml();
            sendHeader(sockOut, 200, htmlPage.length(), "text/html");
            sockOut.writeBytes(htmlPage);
	}catch (IOException e) { e.printStackTrace(); }
	    
    }
  
    //Pre:  DataOutputStream is a socket connection to the browser
    //Post: Produces and sends the (extremely complicated) Admin Page.
    public void sendAdminPage(DataOutputStream sockOut) throws IOException{
	Page page = new Page("Administration Page");
	Form form1 = new Form("POST", "/admin?opt=add");
	Form form2 = new Form("POST", "/admin?opt=del");
	Form formPassword = new Form("POST", "/admin?opt=passchange");
	Form formClean = new Form("POST", "/admin?opt=clean");
	Form formAdminOptions = new Form("POST", "/admin?opt=admin");
	Form formCrawler = new Form("POST", "/admin?opt=crawler");
	Form formKill = new Form("POST", "/admin?opt=kill");
	Input site = new Input("text","sitename","");
	Input url = new Input("text", "url","");
	Input preWord = new Input("text", "preword","");
	Input postWord = new Input("text", "postword", "");
	Input preDef = new Input("text", "predef", "");
	Input postDef = new Input("text", "postdef", "");
	Input higher = new Input("checkbox", "higher", "1");
	Input button = new Input("submit","submit","Add Site");
	Input clean = new Input("submit", "submit", "Clean Database");
	Input dates = new Input("text","cleanperiod","15");
	Input oldPass = new Input("password","oldpass","");
	Input newPass1 = new Input("password","newpass1","");
	Input newPass2 = new Input("password","newpass2","");
	Input btnPassword = new Input("submit","submit","Change Password");
	Input txtTimeout = new Input("text", "timeout", "");
	Input txtFrequency = new Input("text","frequency","");
	Input chkLogging = new Input("checkbox","logging", "1");
	Input btnAdmin = new Input("submit","submit","Submit Changes");
	Input btnKill = new Input("submit","submit","Kill Server");
	Table inputTable = new Table();
	TableCell currCell;
	TableRow currRow;
	DataGrid currentSites = new DataGrid();
	DataGrid currentDates = new DataGrid();
	Object[][] siteList = objSQL.cc_site_list();
	Object[][] dateList = objSQL.cc_admin_get_dates();
	Object[][] adminOpt = objSQL.cc_admin_list();
	boolean isCrawlerRunning = objSQL.cc_crawl_crawler_running();
	boolean isParserRunning = objSQL.cc_crawl_parser_running();
	Link lnkLogout = new Link();
	lnkLogout.add("Log Out");
	lnkLogout.setHref("/admin?opt=LOG");

	chkLogging.checked(adminOpt[1][1].toString().equals("1"));
	txtTimeout.setValue(adminOpt[1][0].toString());
	txtFrequency.setValue(adminOpt[1][2].toString());

	inputTable.setCSSClass("admin");
	currRow = new TableRow();
	currCell = new TableCell();
	currCell.add("Logging");
	currRow.addCell(currCell);
	currCell = new TableCell();
	currCell.add(chkLogging);
	currRow.addCell(currCell);
	inputTable.addRow(currRow);

	currRow = new TableRow();
	currCell = new TableCell();
	currCell.add("Login Timeout (Minutes)");
	currRow.addCell(currCell);
	currCell = new TableCell();
	currCell.add(txtTimeout);
	currRow.addCell(currCell);
	inputTable.addRow(currRow);

	currRow = new TableRow();
	currCell = new TableCell();
	currCell.add("Crawl Frequency (Days)");
	currRow.addCell(currCell);
	currCell = new TableCell();
	currCell.add(txtFrequency);
	currRow.addCell(currCell);
	inputTable.addRow(currRow);

	currRow = new TableRow();
	currCell = new TableCell();
	currCell.add("Crawler Status");
	currRow.addCell(currCell);
	currCell = new TableCell();
	if(isCrawlerRunning)
	    currCell.add("<span class=\"true\">RUNNING</span>");
	else
	    currCell.add("<span class=\"false\">NOT RUNNING</span>");
	currRow.addCell(currCell);
	inputTable.addRow(currRow);

	currRow = new TableRow();
	currCell = new TableCell();
	currCell.add("Parser Status");
	currRow.addCell(currCell);
	currCell = new TableCell();
	if(isParserRunning)
	    currCell.add("<span class=\"true\">RUNNING</span>");
	else
	    currCell.add("<span class=\"false\">NOT RUNNING</span>");
	currRow.addCell(currCell);
	inputTable.addRow(currRow);

	currRow = new TableRow();
	currCell = new TableCell();
	currCell.setColSpan("2");
	currCell.add(btnAdmin);
	currRow.addCell(currCell);
	inputTable.addRow(currRow);
	formAdminOptions.add(inputTable);
	
	inputTable = new Table();
	inputTable.setCSSClass("admin");
	formClean.add(clean);
	dates.setLength(3);
	dates.setSize(5);
	formClean.add(dates);
	formClean.add("<label for=\"cleanperiod\">Days</label>");

	currRow = new TableRow();
	currCell = new TableCell();
	currCell.add("Enter old password:");
	currRow.addCell(currCell);
	currCell = new TableCell();
	currCell.add(oldPass);
	currRow.addCell(currCell);
	inputTable.addRow(currRow);

	currRow = new TableRow();
	currCell = new TableCell();
	currCell.add("Enter new password:");
	currRow.addCell(currCell);
	currCell = new TableCell();
	currCell.add(newPass1);
	currRow.addCell(currCell);
	inputTable.addRow(currRow);

	currRow = new TableRow();
	currCell = new TableCell();
	currCell.add("Confirm new password:");
	currRow.addCell(currCell);
	currCell = new TableCell();
	currCell.add(newPass2);
	currRow.addCell(currCell);
	inputTable.addRow(currRow);

	currRow = new TableRow();
	currCell = new TableCell();
	currCell.add(btnPassword);
	currCell.setColSpan("2");
	currRow.addCell(currCell);
	inputTable.addRow(currRow);
	formPassword.add("<hr>");
	formPassword.add("<h2>Change Password</h2>");
	formPassword.add(inputTable);


	inputTable = new Table();

	page.setJavaScript("function verify(sitename,url,preword,predef,postdef) { if(sitename.value == \"\" || " +
		       " url.value == \"\" || preword.value == \"\" || predef.value == \"\" || " + 
		       "postdef.value == \"\") {  alert(\"Required Values Missing!\"); return false; } return true;}");
	site.setLength(50);
	url.setLength(100);

	inputTable.setCSSClass("inputTable");

	currRow = new TableRow();
	currCell = new TableCell();
	currCell.add("Site name:");
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add(site);
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add("Pre-Word HTML:");
	currRow.addCell(currCell);
	
	currCell = new TableCell();
	currCell.add(preWord);
	currRow.addCell(currCell);
	inputTable.addRow(currRow);

	currRow = new TableRow();
	currCell = new TableCell();
	currCell.add("Site URL:");
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add(url);
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add("Post-Word HTML:");
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add(postWord);
	currRow.addCell(currCell);
	inputTable.addRow(currRow);


	currRow = new TableRow();
	currCell = new TableCell();
	currCell.add("Allow Higher:");
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add(higher);
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add("Pre-Definition HTML:");
	currRow.addCell(currCell);
	
	currCell = new TableCell();
	currCell.add(preDef);
	currRow.addCell(currCell);
	inputTable.addRow(currRow);

	currRow = new TableRow();
	currCell = new TableCell();
	currCell.add("<hr>"); //"Allow External Links:");
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add("<hr>");
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add("Post-Definition HTML:");
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add(postDef);
	currRow.addCell(currCell);
	inputTable.addRow(currRow);

	currRow = new TableRow();
	currCell = new TableCell();
	button.setOnClick("return verify(sitename,url,preword,predef,postdef);");
	currCell.add(button);
	currCell.setColSpan("4");
	currRow.addCell(currCell);
	inputTable.addRow(currRow);

	page.addLink("stylesheet","result.css");
	page.add("<h1>Administration Options</h1>");
	page.add("<hr>");
	page.add(lnkLogout);
	page.add("<hr>");

	form1.add("<h2>Add Site</h2>");
	form1.add(inputTable);
	form1.add("<hr>");
	form1.add(lnkLogout);
	
	Object[][] outTable = new Object[Array.getLength(siteList)][4];
	outTable[0][0] = siteList[0][0];
	outTable[0][1] = siteList[0][1];
	outTable[0][2] = "Delete";
	outTable[0][3] = "Edit";
	for(int i = 1; i < Array.getLength(siteList); i++){
	    outTable[i][0] = siteList[i][0];
	    outTable[i][1] = siteList[i][1];
	    Input delButton = new Input("submit", "", "Delete");
	    //System.out.println((String)outTable[i][0]);
	    Input siteName = new Input("hidden", "sitename", (String)outTable[i][0]);
	    Form delForm = new Form("POST", "/admin?opt=del");
	    delButton.setOnClick("return confirm('Are you sure you want to delete " + 
				 outTable[i][0].toString().replace('\'',' ') + "?');");
	    delForm.add(delButton);
	    delForm.add(siteName);
	    outTable[i][2] = delForm;
	    Input btnEdit = new Input("submit","","Edit");
	    btnEdit.setOnClick("window.location = '/admin?opt=edit&sitename=" + outTable[i][0] + "';");
	    outTable[i][3] = btnEdit;
	}
	currentSites.setFirstRowTitle(true);
	currentSites.setDataSource(outTable);
	currentSites.setCSSClass("admin");

	if(dateList[1][0] != null)
	    dateList[1][0] = dateList[1][0].toString() + "<br>&nbsp;";
	else
	    dateList[1][0] = "<hr>&nbsp;";

	if(dateList[1][1] != null)
	    dateList[1][1] = dateList[1][1].toString() + "<br>&nbsp;";
	else
	    dateList[1][1] = "<hr>&nbsp;";

	if(dateList[1][2] != null)
	    dateList[1][2] = dateList[1][2].toString() + formClean.toString();
	else
	    dateList[1][2] = "<hr>" + formClean.toString();
       
	
	currentDates.setDataSource(dateList);
	currentDates.setFirstRowTitle(true);
	currentDates.setCSSClass("admin");

	btnKill.setOnClick("return confirm('Are you sure?  If you kill the server any crawl/parse in progress will " +
			   "also be terminated.  Also, you WILL need to restart the server manually.');");

	formKill.add("<h2>Server Control</h2>");
	formKill.add("<input type=\"hidden\" name=\"opt\" value=\"KILL\"/>");
	formKill.add(btnKill);
	formKill.add("<hr>");
	
	page.add(formKill);
	page.add("<h2>Administrative Options</h2>");
     	page.add(formAdminOptions);
	page.add("<hr>");
	page.add(currentDates);
	page.add(formPassword);
	page.add("<hr><h2>Current Site List</h2>");
	page.add(currentSites);
	page.add("<hr>");
	page.add(form1);
	sendHeader(sockOut,200,page.toHtml().length(), "text/html");
	sockOut.writeBytes(page.toHtml());
    }

    //Pre:  qs is a QueryString object, and sockOut is a socket connection to
    //      the browser
    //Post: the edit page entry web page is sent.
    public void sendEditPage(DataOutputStream sockOut, QueryString qs) throws IOException{
	String strName = qs.getValue("sitename");
      
	if (strName == ""){
	    sendErrorPage(sockOut,"<li>Error:  Invalid Site Name</li>");
	    return;
	}
	else 
	    strName = getVerboseStr(strName);

	Object[][] results = objSQL.cc_site_find(strName);
	if (results == null){
	    sendErrorPage(sockOut,"<li>Error:  Invalid Site name</li>");
	    return;
	}

	Page page = new Page("Edit Entry for " + strName);
	Form form = new Form("POST", "/admin?opt=submitEdit");
	Input site = new Input("text","sitename",results[1][0].toString().replaceAll("\"","&quot;"));
	Input oldSite = new Input("hidden","oldsitename",results[1][0].toString());
	Input url = new Input("text", "url",results[1][1].toString());
	Input preWord = new Input("text", "preword",results[1][3].toString().replaceAll("\"","&quot;"));
	Input postWord = new Input("text", "postword",results[1][4].toString().replaceAll("\"","&quot;") );
	Input preDef = new Input("text", "predef", results[1][5].toString().replaceAll("\"","&quot;"));
	Input postDef = new Input("text", "postdef", results[1][6].toString().replaceAll("\"","&quot;"));
	Input higher = new Input("checkbox", "higher", "1");
	if(results[1][2].toString().equals("1"))
	    higher.checked(true);
	//Input external = new Input("checkBox", "external", "1");
	//if(results[1][3].toString().equals("1"))
	  //  external.checked(true);
	Input button = new Input("submit","submit","Submit Changes");
	Input cancel = new Input("submit","submit","Cancel");
	Table inputTable = new Table();
	TableCell currCell;
	TableRow currRow;
	
	page.setJavaScript("function verify(sitename,url,preword,predef,postdef) { if(sitename.value == \"\" || " +
		       " url.value == \"\" || preword.value == \"\" || predef.value == \"\" || " + 
		       "postdef.value == \"\") {  alert(\"Required Values Missing!\"); return false; } return true;}");
	site.setLength(50);
	url.setLength(100);

	inputTable.setCSSClass("inputTable");

	currRow = new TableRow();
	currCell = new TableCell();
	currCell.setColSpan("4");
	currCell.setHeader(true);
	currCell.add("<h2>Edit Site</h2>");
	currRow.addCell(currCell);
	inputTable.addRow(currRow);

	currRow = new TableRow();
	currCell = new TableCell();
	currCell.add("Site name:");
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add(site);
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add("Pre-Word HTML:");
	currRow.addCell(currCell);
	
	currCell = new TableCell();
	currCell.add(preWord);
	currRow.addCell(currCell);
	inputTable.addRow(currRow);

	currRow = new TableRow();
	currCell = new TableCell();
	currCell.add("Site URL:");
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add(url);
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add("Post-Word HTML:");
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add(postWord);
	currRow.addCell(currCell);
	inputTable.addRow(currRow);


	currRow = new TableRow();
	currCell = new TableCell();
	currCell.add("Allow Higher:");
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add(higher);
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add("Pre-Definition HTML:");
	currRow.addCell(currCell);
	
	currCell = new TableCell();
	currCell.add(preDef);
	currRow.addCell(currCell);
	inputTable.addRow(currRow);

	currRow = new TableRow();
	currCell = new TableCell();
	currCell.add("<hr>");//"Allow External Links:");
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add("<hr>");
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add("Post-Definition HTML:");
	currRow.addCell(currCell);

	currCell = new TableCell();
	currCell.add(postDef);
	currRow.addCell(currCell);
	inputTable.addRow(currRow);

	currRow = new TableRow();
	currCell = new TableCell();
	button.setOnClick("return (verify(sitename,url,preword,predef,postdef) && confirm('Are you sure?'));");
	cancel.setOnClick("window.location = '/admin'; return false;");
	currCell.add(button.toString() + cancel.toString() + oldSite.toString());
	currCell.setColSpan("4");
	currRow.addCell(currCell);
	inputTable.addRow(currRow);
	form.add(inputTable);
	page.add(form);
	page.addLink("stylesheet","result.css");
	sendHeader(sockOut,200,page.toHtml().length(), "text/html");
	sockOut.writeBytes(page.toHtml());
    }

    //Pre:  sockOut is a socket connection to the browser, strOther is the
    //      error message(s)
    //Post: the error page is sent to the browser
    public void sendErrorPage(DataOutputStream sockOut, String strOther ) throws IOException{

	Page errorPage = new Page("Input Error(s)");
	
	errorPage.addLink("stylesheet","result.css");
	errorPage.add("<h1>Input Errors</h1><hr><h2>The Following Errors Were Found:</h2><br><br>");
	errorPage.add(strOther);
	errorPage.add("<br>");
	errorPage.add("<a href=\"/admin\"> Click here to return</a>");

	sendHeader(sockOut,200,errorPage.toHtml().length(),"text/html");
	sockOut.writeBytes(errorPage.toHtml());
    }
    
    //Pre:  sockOut is a socket connection to the browser
    //Post: the login page is sent to the browser
    public void sendLoginPage(DataOutputStream sockOut) throws IOException{
	Page loginPage = new Page("Administration Login");
	Form loginForm = new Form("POST", "/admin");
	Input iptPwd = new Input("password", "password", "");
	Input iptSubmit = new Input("submit", "submit", "Log In");
	
	loginPage.setJavaScript("function sf() { document.f.password.focus(); } ");
	loginPage.setOnBodyLoad("sf();");
	loginForm.setCSSClass("input");
	loginPage.addLink("stylesheet", "result.css");
	loginForm.add(iptPwd);
	loginForm.add(iptSubmit);
	loginForm.setName("f");

	loginPage.add("<h1>Administration Login</h1><hr>");
	loginPage.add(loginForm);
	sendHeader(sockOut,200,loginPage.toHtml().length(),"text/html");
	sockOut.writeBytes(loginPage.toHtml());
    }

    //Pre:  sockOut is a socket connection to the browser, pdData is the post
    //      data
    //Post: the password change is processed
    public void processLoginChange(DataOutputStream sockOut, PostData pdData) throws IOException{

	String oldPword = pdData.getValue("oldpass");
	String newPword1 = pdData.getValue("newpass1");
	String newPword2 = pdData.getValue("newpass2");

	if(oldPword.equals("") || newPword1.equals("") 
	   || newPword2.equals("")){
	    sendErrorPage(sockOut,"<li>Error:  Insufficient Data Entered</li>");
	    return;
	}
	else if (!newPword2.equals(newPword1)){
	    sendErrorPage(sockOut,"<li>Error:  New Password Entries Don't Match");
	    return;
	}
	else if(!isAlphaNumeric(newPword1) || !isAlphaNumeric(newPword2) ||
		!isAlphaNumeric(oldPword)){
	    sendErrorPage(sockOut,"<li>Error:  Invalid Password(s) Entered!  " + 
			  "Passwords may only be alpha-numeric");
	    return;
	}
	else if(newPword1.length() < 5 || newPword2.length() < 5){
	    sendErrorPage(sockOut,"<li>Error:  New Passwords must be at least 5 characters");
	    return;
	}
	else if(!objSQL.cc_admin_change_pass(oldPword, newPword1)){
	    sendErrorPage(sockOut,"<li>Error:  Old Password is Invalid");
	    return;
	}

	Page page = new Page("Password Change");
	Link link = new Link();
	
	link.setHref("/admin");
	link.add("Return to Administration");
	
	page.add("<h2>Password Changed Successfully</h2>");
	page.add(link);
	
	sendHeader(sockOut,200,page.toHtml().length(),"text/html");
	sockOut.writeBytes(page.toHtml());
    }

    //Pre:  sockOut is a socket connection to the browser, pdData is the post
    //      data
    //Post: processes the changing of administrative options
    public void processAdminOpt(DataOutputStream sockOut, PostData pdData) throws IOException{
	String timeout = pdData.getValue("timeout");
	String frequency = pdData.getValue("frequency");
	String strLogging = pdData.getValue("logging");
	Object[][] current = objSQL.cc_admin_list();
	String strResult = "";
	int intTimeout = Integer.parseInt(current[1][0].toString());
	int intFrequency = Integer.parseInt(current[1][2].toString());

	boolean logging = strLogging.equals("1");
	
	if(!isNumeric(timeout)){
	    strResult = strResult + "<li>Timeout not a valid integer!</li>";
	}
	else{
	    intTimeout = Integer.parseInt(timeout);
	    strResult = strResult + "<li>Timeout changed successfully</li>";
	}

	if(!isNumeric(frequency)){
	    strResult = strResult + "<li>Frequency not a valid integer!</li>";
	}
	else{
	    strResult = strResult + "<li>Frequency changed succesfully</li>";
	    intFrequency = Integer.parseInt(frequency);
	}

	strResult = strResult + "<li>Logging updated successfully</li>";

	objSQL.cc_admin_edit(intTimeout,logging,intFrequency);

	Page page = new Page("Admin Options Set");
	Link link = new Link();
	link.setHref("/admin");
	link.add("Return to Administration");

	page.add("<h1>Admin Options Modified</h1>");
	page.add(strResult + "<br>");

	page.add(link);
	sendHeader(sockOut,200,page.toHtml().length(),"text/html");
	sockOut.writeBytes(page.toHtml());
    }

    //Pre:  sockOut is a socket connection to the browser, pdData is the post
    //      data
    //Post: the database is cleaned for the number of specified days, or for 15
    //      the input is invalid
    public void processClean(DataOutputStream sockOut, PostData pdData) throws IOException{
	String days = pdData.getValue("cleanperiod");
	int intDays = 15;

	if(isNumeric(days))
	    intDays = Integer.parseInt(days);
	
	objSQL.cc_search_clean(intDays);
	
	Page page = new Page("Clean Successful");
	Link link = new Link();

	link.setHref("/admin");
	link.add("Return to Administration");

	page.add("<h2>Database Cleaned Successfully(" + intDays + " Days)</h2><br>");
	page.add(link);

	sendHeader(sockOut,200,page.toHtml().length(),"text/html");
	sockOut.writeBytes(page.toHtml());
    }

    //Pre:  inStr is non-null
    //Post: returns false if any characters are non-alphanumeric, true otherwise
    private boolean isAlphaNumeric(String inStr){
	for(int i = 0; i < inStr.length(); i++){
	    if(!Character.isLetter(inStr.charAt(i)) && 
	       !Character.isDigit(inStr.charAt(i)))
		return false;
	}
	return true;
    }

    //Pre:  inStr is non-null
    //Post: returns true if the str contains only numbers, false otherwise
    private boolean isNumeric(String inStr){
	for(int i = 0; i < inStr.length(); i++){
	    if(!Character.isDigit(inStr.charAt(i)))
		return false;
	}
	return true;
    }

    //Pre:  inStr is non-null
    //Post: strips out all characters that are not letters or numbers
    private String makeAlphaNumeric(String inStr){
	for(int i = 0; i < inStr.length(); i++){
	    if(!Character.isLetter(inStr.charAt(i)) &&
	       !Character.isDigit(inStr.charAt(i)) &&
	       inStr.charAt(i) != ' '){
		inStr = inStr.replaceAll("" + inStr.charAt(i),"");
	    }
	}
	return inStr;
    }


}
