package Html_Engine;

public class Input extends Control {
    
	//The type of the input
    private String strType;

	//value attribute of the tag
    private String strValue;

	//src attribute of the tag
    private String strSrc;

	//name of the item
    private String strName;

	//alternate text for the item
    private String strAlt;

	//size for input boxes
    private int intSize;

	//max length for the input
    private int intLength;

	//read only bool
    private boolean boolReadOnly;

	//T/F checked boolean
    private boolean boolChecked;
    
	//Pre: inType is a valid type string, inName is a valid string for the name, inValue is a valid value or empty
	//Post:  the item is initialized
    public Input(String inType, String inName, String inValue){
        strType = inType;
        if (!inName.equals("submit"))
            strName = inName;
        else
            strName = "";
        strValue = inValue;
        strSrc = "";
        strAlt = "";
    }
    
	//Pre:  inPic is an Html image tag, inName is a valid string for the name, inValue is a valid value or empty
	//Post:  an image input is initialized
    public Input(Image inPic, String inName, String inValue){
        strType = "image";
        strName = inName;
        strSrc = inPic.getSrc();
        strAlt = inPic.getAlt();
        strValue = inValue;
    }
    
	//Pre:  inSize is a non-zero int
	//Post:  the size of the input is set to inSize
    public void setSize(int inSize){
        intSize = inSize;
    }
    
	//Pre:  inLength is a non-zero int
	//Post:  the max length of the input is set to inLength
    public void setLength(int inLength){
        intLength = inLength;
    }
    
	//Pre:  none
	//Post:  boolReadOnly is set to inValue
    public void readOnly(boolean inValue){
        boolReadOnly = inValue;
    }
    
	//Pre:  none
	//Post:  strValue is set to inValue
    public void setValue(String inValue)
	{
	strValue = inValue;
    }
    
	//Pre:  none
	//Post:  boolChecked is set to inValue
    public void checked(boolean inValue)
	{
        boolChecked = inValue;
    }
    
	//Pre:  none
	//Post:  returns the string equivalent of the Input
    public String toString() 
	{
        String input = "<input " + getStandardAttributes();
        
        input = input + "type=\"" + strType + "\" ";
        
        if(!strName.equals(""))
            input = input + "name=\"" + strName + "\" ";
        
        if(strType.equals("image"))
            input = input + "src=\"" + strSrc + "\" alt=\"" + strAlt + "\" ";
        else if( (strType.equals("checkbox") || strType.equals("radio")) && boolChecked)
            input = input + "checked=\"true\" ";
        else if(strType.equals("text") && intLength > 0)
            input = input + "maxlength=\"" + intLength + "\" ";
        
        if(boolReadOnly)
            input = input + "readonly=\"readonly\" ";
        
        if(intSize > 0)
            input = input + "size=\"" + intSize + "\" ";
        input = input + "value=\"" + strValue + "\" />";
        return input;
    }

}
