import java.io.*;
import java.util.*;
import java.sql.*;
import java.net.*;
import java.lang.reflect.Array;

/*
 * Created on Oct 22, 2004
 * By Evan Kaiser, Don Frehulfer, and Matt Berntsen
 * CS 479 - Capstone Design
 */

//public class Parser extends Thread{
public class Parser extends TimerTask{
    public final static int PORT = 80; //default web port
    public final static String protocol = "http://";

    private cc_db db;              //Database
    private PageContentList pList; //List of Pages
    private HttpURLConnection uC;  //Connection to current url
    private URL u; //URL is in the form "http://www.yahoo.com/"

    //Pre:  inList is a reference to a PageContentList created in RunCrawler.
    //Post: db gets instantiated and pList gets set to inList.
    public Parser(PageContentList inList){
	db = new cc_db();
	pList = inList;
    }

    //Pre:  inURL has a value of the form http://domain.ext/optional_path.
    //Post: Return true if connection was made.
    private boolean connectToSite(String inURL){
	int responseCode = 0;

	try{
	    u = new URL(inURL);
	    //	    uC.setChunkedStreamingMode(32);
	    // connet to the site
	    uC = (HttpURLConnection)u.openConnection();
	    responseCode = uC.getResponseCode();
	}
	catch (Exception ex){
	    return false;
	}

	if (responseCode != HttpURLConnection.HTTP_OK){
	    return false;
	}
	return true;
    }

    //Pre;  inPos and inSize has a value where 0 <= inPos < inSize.
    //Post: Return next position, with wraparound, of buffer.
    private int iterate(int inPos, int inSize){
	if(inPos >= 0 && inPos < inSize){
	    if(inPos == inSize-1)
		inPos = 0;
	    else
		inPos++;
	    return inPos;
	}
	else
	    return -1;
    }

    //Pre:  inPage is a reference to a PageContent in the PageContentList.
    //Post: Extract page name, url, preterm, postterm, predef, and postdef
    //      from inPage and use to find terms and defs from the page.
    //      Will return in the middle of the function if bad html or end of
    //      page is found.
    private int parseTerms(PageContent inPage){
	int bufLen = 50;
	char buffer[] = new char[bufLen]; //wraparound buffer for parsing
	int bufCount = 0;
	InputStream input;
	int count = 0;
	int termsAdded = 0;
	boolean posttermFound = false;
	boolean predefFound = false;
	boolean postdefFound = false;
	String preterm = inPage.PRETERM;
	String postterm = inPage.POSTTERM;
	String predef = inPage.PREDEF;
	String postdef = inPage.POSTDEF;
	String term, def;
	ArrayList termList, defList;
	term = "";
	def = "";
	termList = new ArrayList(0);
	defList = new ArrayList(0);

	try{
	    int length = 0;
	    boolean more = true;
	    String str = "";
	    input = uC.getInputStream();
	    String lengthStr = uC.getHeaderField("content-length");
	    DataInputStream tempPage = new DataInputStream(input);
	    BufferedWriter outPage = new BufferedWriter(new FileWriter(".parsing2_tmp"));


	    /***********************************
	     * Copy from socket stream to file *
	     ***********************************/
	    boolean readByLine = false;
	    String temp = "";

	    if(lengthStr != null){
		byte[] data = new byte[length];
		length = Integer.parseInt(lengthStr.trim());
		
		tempPage.readFully(data);
		temp = new String(data);
		if(temp.length() == length){
		    temp = temp.replace('\n', ' ');
		    temp = temp.replace('\t', ' ');
		}
		else
		    readByLine = true;
	    }

	    if(lengthStr == null || readByLine){
		BufferedReader bufPage = new BufferedReader(new InputStreamReader(input));
		temp = "";
		while(more) { // read until no more
		    str = bufPage.readLine();
		    //System.err.println(str);
		    if(str == null)
			more = false;
		    else{
			temp = temp.replace('\t', ' ');
			temp = temp + str + " ";
		    }
		}
	    }
	    int tempLen = temp.length();
	    while((temp = temp.replaceAll("  ", " ")).length() != tempLen){
		//System.err.println(tempLen);
		//System.err.println(temp.length());
		tempLen = temp.length();
	    }
	    outPage.write(temp);
	    outPage.write('>'); //to avoid infinite loops in case of bad html
	    outPage.flush();

	    FileReader page = new FileReader(".parsing2_tmp");

	    //parse for terms
	    while(page.ready()){
		
		buffer[bufCount] = (char)page.read();
		if(!page.ready())
		    return termsAdded;
		bufCount = iterate(bufCount, bufLen);
		
		//if(inName.equals("Enterasys Glossary - W"))
		//if(inPage.URL.equals("www.cabletron.com/training/certification/courses/practice/index.html"))
		//if(inPage.URL.indexOf("www.cabletron.com") >= 0)
		//System.err.print("\nCrap or preterm 0: "+buffer[bufCount]);

		/********************************
		 * Look for preterm             *
		 ********************************/

		if(Character.toLowerCase(buffer[bufCount]) == 
		   Character.toLowerCase(preterm.charAt(0))){
		    count = 0;
		    while((count < preterm.length()-1) && 
			  (Character.toLowerCase(buffer[bufCount]) == 
			   Character.toLowerCase(preterm.charAt(count)))){
			buffer[bufCount] = (char)page.read();
			if(!page.ready())
			    return termsAdded;
			bufCount = iterate(bufCount, bufLen);
			count++;
			if(buffer[bufCount] == '\"'){ //ignore tag characters between quotes
			    do{
				buffer[bufCount] = (char)page.read();
				if(!page.ready())
				    return termsAdded;
				bufCount = iterate(bufCount, bufLen);
			    }
			    while(buffer[bufCount] != '\"');
			}
			if(preterm.charAt(count) == '\"'){
			    do{
				count++;
			    }
			    while(count < preterm.length() && preterm.charAt(count) != '\"');
			}
			//if(inName.equals("Enterasys Glossary - W"))
			//if(inPage.URL.equals("www.cabletron.com/training/certification/courses/practice/index.html"))
			/*
			if(inPage.URL.indexOf("www.cabletron.com") >= 0){
			    System.err.print("\nPreterm "+count+": "+buffer[bufCount]);
			    System.err.print(", and character it's being compared to: " + preterm.charAt(count));
			    System.err.print(", and preterm length - 1: " + (preterm.length()-1));
			    }*/
		    }

		    if(count == preterm.length())
			return termsAdded;
		    
		    if((count == preterm.length()-1) && 
		       (Character.toLowerCase(buffer[bufCount]) == 
			Character.toLowerCase(preterm.charAt(preterm.length()-1)))){ //found a preterm
			while(!predefFound){
			    buffer[bufCount] = (char)page.read();
			    if(!page.ready())
				return termsAdded;
			    bufCount = iterate(bufCount, bufLen);
			    
			   
			    //if(inName.equals("Enterasys Glossary - W"))
			    //if(inPage.URL.equals("www.cabletron.com/training/certification/courses/practice/index.html"))
			    //if(inPage.URL.indexOf("www.cabletron.com") >= 0)
			    //System.err.print("\nPostterm 0 or predef 0: "+buffer[bufCount]);
			    
			    if(!posttermFound && (postterm.length() > 0) && 
			       (Character.toLowerCase(buffer[bufCount]) == 
				Character.toLowerCase(postterm.charAt(0)))){ 
				//optional postterm possibly exists
				count = 0;

				/********************************
				 * Look for postterm            *
				 ********************************/
				
				while((count < postterm.length()-1) && 
				      (Character.toLowerCase(buffer[bufCount]) == 
				       Character.toLowerCase(postterm.charAt(count)))){
				    buffer[bufCount] = (char)page.read();
				    if(!page.ready())
					return termsAdded;
				    bufCount = iterate(bufCount, bufLen);
				    count++;
				    if(buffer[bufCount] == '\"'){ //ignore tag characters between quotes
					do{
					    buffer[bufCount] = (char)page.read();
					    if(!page.ready())
						return termsAdded;
					    bufCount = iterate(bufCount, bufLen);
					}
					while(buffer[bufCount] != '\"');
				    }
				    if(postterm.charAt(count) == '\"'){
					do{
					    count++;
					}
					while(count < postterm.length() && postterm.charAt(count) != '\"');
				    }
				    //if(inPage.URL.equals("www.cabletron.com/training/certification/courses/practice/index.html"))
				    //if(inName.equals("Enterasys Glossary - W"))
				    //if(inPage.URL.indexOf("www.cabletron.com") >= 0)
				    //System.err.print("\nPostterm "+count+": "+buffer[bufCount]);
				    
				}

				if(count == postterm.length())
				    return termsAdded;
				
				if((count == postterm.length()-1) && 
				   (Character.toLowerCase(buffer[bufCount]) == 
				    Character.toLowerCase(postterm.charAt(postterm.length()-1)))){
				    posttermFound = true;
				}
				else{//not a postterm, ignore everything until '>'
				    while(buffer[bufCount] != '>'){
					buffer[bufCount] = (char)page.read();
					if(!page.ready())
					    return termsAdded;
					bufCount = iterate(bufCount, bufLen);
					
				    }
				}
			    }
			    else if(Character.toLowerCase(buffer[bufCount]) == 
				    Character.toLowerCase(predef.charAt(0))){ //possibly found predef
				count = 0;

				/********************************
				 * Look for predef              *
				 ********************************/
				
				while((count < predef.length()-1) && 
				      (Character.toLowerCase(buffer[bufCount]) == 
				       Character.toLowerCase(predef.charAt(count)))){
				    buffer[bufCount] = (char)page.read();
				    if(!page.ready())
					return termsAdded;
				    bufCount = iterate(bufCount, bufLen);
				    count++;
				    if(buffer[bufCount] == '\"'){ //ignore tag characters between quotes
					do{
					    buffer[bufCount] = (char)page.read();
					    if(!page.ready())
						return termsAdded;
					    bufCount = iterate(bufCount, bufLen);
					}
					while(buffer[bufCount] != '\"');
				    }
				    if(predef.charAt(count) == '\"'){
					do{
					    count++;
					}
					while(count < predef.length() && predef.charAt(count) != '\"');
				    }
				    //if(inName.equals("Enterasys Glossary - W"))
				    //if(inPage.URL.equals("www.cabletron.com/training/certification/courses/practice/index.html"))

				    //if(inPage.URL.indexOf("www.cabletron.com") >= 0)
				    //System.err.print("\nPredef "+count+": "+buffer[bufCount]);
				    
				}

				if(count == predef.length())
				    return termsAdded;
				
				if((count == predef.length()-1) && 
				   (Character.toLowerCase(buffer[bufCount]) == 
				    Character.toLowerCase(predef.charAt(predef.length()-1)))){
				    predefFound = true;
				}
				else{//not a predef, ignore everything until '>'
				    while(buffer[bufCount] != '>'){
					buffer[bufCount] = (char)page.read();
					if(!page.ready())
					    return termsAdded;
					bufCount = iterate(bufCount, bufLen);
					
				    }
				}
			    }
			    else if(buffer[bufCount] == '<'){ //don't want tags in term
				buffer[bufCount] = (char)page.read();
				if(!page.ready())
				    return termsAdded;
				bufCount = iterate(bufCount, bufLen);
				
				while(buffer[bufCount] != '>'){ //ignore tag
				    buffer[bufCount] = (char)page.read();
				    if(!page.ready())
					return termsAdded;
				    bufCount = iterate(bufCount, bufLen);
				    
				}
				
			    }
			    else{
				if(!posttermFound){
				    term += buffer[bufCount]; //add new character to term
				    //if(inName.equals("Enterasys Glossary - W"))
				    //if(inPage.URL.equals("www.cabletron.com/training/certification/courses/practice/index.html"))
				    
				    //if(inPage.URL.indexOf("www.cabletron.com") >= 0)
				    //System.err.print("\nTerm: "+buffer[bufCount]);
				}
			    }
			}
			while(!postdefFound){
			    buffer[bufCount] = (char)page.read();
			    if(!page.ready())
				return termsAdded;
			    bufCount = iterate(bufCount, bufLen);
			    
			    
			    if(Character.toLowerCase(buffer[bufCount]) == 
			       Character.toLowerCase(postdef.charAt(0))){ //possibly found postdef
				count = 0;

				/********************************
				 * Look for postdef             *
				 ********************************/
				
				while((count < postdef.length()-1) && 
				      (Character.toLowerCase(buffer[bufCount]) == 
				       Character.toLowerCase(postdef.charAt(count)))){
				    buffer[bufCount] = (char)page.read();
				    if(!page.ready())
					return termsAdded;
				    bufCount = iterate(bufCount, bufLen);
				    count++;
				    if(buffer[bufCount] == '\"'){ //ignore tag characters between quotes
					do{
					    buffer[bufCount] = (char)page.read();
					    if(!page.ready())
						return termsAdded;
					    bufCount = iterate(bufCount, bufLen);
					}
					while(buffer[bufCount] != '\"');
				    }
				    if(postdef.charAt(count) == '\"'){
					do{
					    count++;
					}
					while(count < postdef.length() && postdef.charAt(count) != '\"');
				    }
				    //if(inName.equals("Enterasys Glossary - W"))
				    //if(inPage.URL.equals("www.cabletron.com/training/certification/courses/practice/index.html"))
				    //if(inPage.URL.indexOf("www.cabletron.com") >= 0)
				    //System.err.print("\nPostdef "+count+": "+buffer[bufCount]);
				    
				}

				if(count == postdef.length())
				    return termsAdded;
				
				if((count == postdef.length()-1) && 
				   (Character.toLowerCase(buffer[bufCount]) == 
				    Character.toLowerCase(postdef.charAt(postdef.length()-1)))){
				    postdefFound = true;
				}
				else{//not a postterm, ignore everything until '>' 
				    while(buffer[bufCount] != '>'){
					buffer[bufCount] = (char)page.read();
					if(!page.ready())
					    return termsAdded;
					bufCount = iterate(bufCount, bufLen);
					
				    }
				}
			    }
			    else if(buffer[bufCount] == '<'){ //don't want tags in definition
				buffer[bufCount] = (char)page.read();
				if(!page.ready())
				    return termsAdded;
				bufCount = iterate(bufCount, bufLen);
				
				while(buffer[bufCount] != '>'){ //ignore tag
				    buffer[bufCount] = (char)page.read();
				    if(!page.ready())
					return termsAdded;
				    bufCount = iterate(bufCount, bufLen);
				    
				}
				
			    }
			    else{
				if(!postdefFound){
				    def += buffer[bufCount]; //add new character to def
				    //if(inName.equals("Enterasys Glossary - W"))
				    //if(inPage.URL.equals("www.cabletron.com/training/certification/courses/practice/index.html"))
				    //if(inPage.URL.indexOf("www.cabletron.com") >= 0)
				    //System.err.print("\nDef: "+buffer[bufCount]);
				}
			    }
			}

			/********************************
			 * Everything has been found    *
			 ********************************/
			//add values to arraylists
			
			term = term.trim();
			def = def.trim();

			//			if(inPage.URL.equals("www.cabletron.com/training/certification/courses/practice/index.html")){
			/*
			if(inPage.URL.indexOf("www.cabletron.com") >= 0){
			    System.err.println("\nTerm: "+term);
			    System.err.println("\nDef: "+def);
			    System.err.println("\nName: "+inPage.NAME);
			    System.err.println("\nURL: "+inPage.URL);
			    }*/
			//			}
			//if(inPage.NAME.equals("Webopedia"))
			//System.out.println("Term: " + term.replace('\"', '\'') + "\nDef: " + def.replace('\"','\''));
			db.cc_search_add(term.replace('\"', '\'').replace('\\', '/'),
					 def.replace('\"','\'').replace('\\', '/'),
					 inPage.NAME, inPage.URL);
			termsAdded++;
			//reset term, def, and conditions for next iteration
			term = def = "";
			posttermFound = predefFound = postdefFound = false;
		    }
		}
	    }
	    page.close();
	}
	catch (IOException ex){
	    return termsAdded;
	}
	return termsAdded;
    }

    //Pre:  none
    //Post: Removes a page from the PageContentList, attempts to connect to the
    //      url, then calls parseTerms. If a PageContent is found with the name
    //      "Done", then the Crawler is done, so the Thread sleeps.
    public void run(){
	boolean ccthreadDone = false;
	int pageCount = 0;
	int totalTerms = 0;

	//if(connectToSite("http://whatis.techtarget.com/definition/0,,sid9_gci213035,00.html")){
	//parseTerms(new PageContent("Whatis?com", "http://whatis.techtarget.com/definition/0,,sid9_gci213035,00.html", "<!-- begin definition --> <font class=\"item\">", "</font><br>", "<img src=\"/images/spacer.gif\" height=\"5\" width=\"1\"><br>", "<!-- end definition -->"));
	//}

	if(db.cc_admin_crawl() && !db.cc_crawl_parser_running()){
	    db.cc_crawl_set_parser(true);

	    while(!ccthreadDone || pList.getSize() > 0){
		if(pList.getSize() > 0){
		    PageContent curPage = (PageContent)pList.getLastSite();
		    if(curPage.NAME.charAt(0) == 4) //found EOT character
			ccthreadDone = true;
		    else{
			
			if(curPage.URL.indexOf(protocol) >= 0){
			    curPage.URL = curPage.URL.substring(protocol.length(),curPage.URL.length());
			}
			
			if(connectToSite("http://" + curPage.URL)){
			    //System.err.println("\nParsing through "+curPage.NAME + " url="+curPage.URL+"\nWith preterm="+
			    //curPage.PRETERM+", postterm="+curPage.POSTTERM+", predef="+
			    //	       curPage.PREDEF+", and postdef="+curPage.POSTDEF);
			    totalTerms += parseTerms(curPage);
			    pageCount++;
			}
		    //else
			    //System.err.println("Failed to connect to http://" + curPage.URL);
		    }
		//System.err.println("Current page list size (Parser): "+pList.getSize());
		//System.err.println("Finished parsing through page.");
		}
	    }
	    System.out.println("Parsed through "+pageCount+" pages.");
	    System.out.println("Found "+totalTerms+" terms.");
	    db.cc_crawl_set_parser(false);
	}
	if(db.cc_crawl_parser_running())
	    System.out.println("Parser is already running.");
    }
}
